#!/usr/bin/python3
# -*- coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2016 Canonical Ltd.
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; version 3.
#
# This program is distributed in the hope that it will be useful, but WITHOUTa
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# this program.  If not, see <http://www.gnu.org/licenses/>.


# Copy ubuntu-system-settings keyboard panel settings from AccountsService to
# GSettings.  This might seem backwards (AS has the more modern location),
# and early on in Ubuntu Touch days, we did indeed only store the values in
# AccountsService.  But once we started using indicator-keyboard, which read
# from GSettings, it was easier to copy the values to both places rather than
# update the indicator and all other legacy consumers of the GSettings key in
# Ubuntu.  So we do a one-time sync here if we have values in AccountsService
# but not GSettings.  Along with a change to our code to write to both places
# when the user adjust settings.


import dbus
import os
import sys

from gi.repository import Gio
from gi.repository import GLib


system_bus = dbus.SystemBus()
manager_proxy = system_bus.get_object('org.freedesktop.Accounts',
                                      '/org/freedesktop/Accounts')
object_path = manager_proxy.FindUserById(
    os.getuid(), dbus_interface='org.freedesktop.Accounts'
)
user_proxy = system_bus.get_object('org.freedesktop.Accounts', object_path)

gsettings = Gio.Settings(schema="org.gnome.desktop.input-sources")


def get_accountsservice():
    try:
        return user_proxy.Get("org.freedesktop.Accounts.User", "InputSources",
                              dbus_interface=dbus.PROPERTIES_IFACE)
    except dbus.exceptions.DBusException as e:
        print("Couldn't get InputSources: %s" % (e), file=sys.stderr)
        return []


def has_gsettings():
    return bool(gsettings.get_value("sources"))


def set_gsettings(as_value):
    # AS stores a list of maps, GSettings stores a list of tuples.
    builder = GLib.VariantBuilder.new(GLib.VariantType.new("a(ss)"))
    for as_map in as_value:
        for as_key in as_map:
            gkey = GLib.Variant.new_string(as_key)
            gvalue = GLib.Variant.new_string(as_map[as_key])
            gtuple = GLib.Variant.new_tuple(gkey, gvalue)
            builder.add_value(gtuple)
    gsettings.set_value("sources", builder.end())


as_value = get_accountsservice()
if as_value and not has_gsettings():
    set_gsettings(as_value)



#!/usr/bin/python3
# -*- coding: utf-8 -*-
# Copyright (C) 2013 Canonical Ltd.
#
# Authors:
#  Iain Lane <iain.lane@canonical.com>
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; version 3.
#
# This program is distributed in the hope that it will be useful, but WITHOUTa
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

# Migrate ubuntu-system-settings sound panel settings from GSettings to
# AccountsService

from __future__ import print_function

import dbus
import os
import sys

from gi.repository import Gio

system_bus = dbus.SystemBus()
manager_proxy = system_bus.get_object('org.freedesktop.Accounts',
                                      '/org/freedesktop/Accounts')
object_path = manager_proxy.FindUserById(
    os.getuid(), dbus_interface='org.freedesktop.Accounts'
)
user_proxy = system_bus.get_object('org.freedesktop.Accounts', object_path)

gsettings = Gio.Settings(schema="com.lomiri.touch.sound")


def get_string(key):
    return gsettings.get_string(key)


def get_bool(key):
    return gsettings.get_boolean(key)


def set_as_setting(interface, setting, value):
    try:
        user_proxy.Set(interface, setting, value,
                       dbus_interface=dbus.PROPERTIES_IFACE)
    except dbus.exceptions.DBusException as e:
        print("Couldn't update %s: %s" % (setting, e), file=sys.stderr)

# (gsettings key, accountsservice key, function to retrieve from gsettings)
keys = [('silent-mode', 'SilentMode', get_bool),
        ('incoming-call-sound', 'IncomingCallSound', get_string),
        ('incoming-message-sound', 'IncomingMessageSound', get_string)]

for (g, a, f) in keys:
    set_as_setting('com.lomiri.touch.AccountsService.Sound', a, f(g))



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2014-2016 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.

import dbus
from time import sleep

from autopilot.introspection.dbus import StateNotFoundError
from autopilot.matchers import Eventually
from testtools.matchers import Equals, raises, StartsWith
from unittest import skip

from lomiri_system_settings.tests import (
    CellularBaseTestCase, RDO_IFACE,
    NETREG_IFACE, ACCOUNTS_PHONE_IFACE, CON_IFACE)

from lomiri_system_settings.tests.connectivity import (
    SIM_IFACE as CTV_SIM_IFACE
)

DEV_IFACE = 'org.freedesktop.NetworkManager.Device'


class CellularTestCase(CellularBaseTestCase):

    def test_enable_data(self):
        self.ctv_private.Set(CON_IFACE, 'MobileDataEnabled', False)
        self.cellular_page.enable_data()
        self.assertThat(
            lambda: self.ctv_private.Get(CON_IFACE, 'MobileDataEnabled'),
            Eventually(Equals(True))
        )

    def test_disable_data(self):
        self.ctv_private.Set(CON_IFACE, 'MobileDataEnabled', True)
        self.cellular_page.disable_data()
        self.assertThat(
            lambda: self.ctv_private.Get(CON_IFACE, 'MobileDataEnabled'),
            Eventually(Equals(False))
        )

    def test_remote_manipulation_of_data(self):
        self.ctv_private.Set(CON_IFACE, 'MobileDataEnabled', True)
        self.assertThat(lambda: self.cellular_page.get_data(),
                        Eventually(Equals(True)))
        sleep(1)
        self.ctv_private.Set(CON_IFACE, 'MobileDataEnabled', False)
        sleep(1)
        self.assertThat(lambda: self.cellular_page.get_data(),
                        Eventually(Equals(False)))

    def test_enable_roaming(self):
        self.ctv_private.Set(CON_IFACE, 'MobileDataEnabled', True)
        self.ctv_sim0.Set(CTV_SIM_IFACE, 'DataRoamingEnabled', False)
        self.cellular_page.enable_roaming()
        self.assertThat(
            lambda: self.ctv_sim0.Get(CTV_SIM_IFACE, 'DataRoamingEnabled'),
            Eventually(Equals(True))
        )

    def test_disable_roaming(self):
        self.ctv_private.Set(CON_IFACE, 'MobileDataEnabled', True)
        self.ctv_sim0.Set(CTV_SIM_IFACE, 'DataRoamingEnabled', True)
        self.cellular_page.disable_roaming()
        self.assertThat(
            lambda: self.ctv_sim0.Get(CTV_SIM_IFACE, 'DataRoamingEnabled'),
            Eventually(Equals(False))
        )

    def test_connection_type(self):

        def get_pref():
            return self.modem_0.Get(RDO_IFACE, 'TechnologyPreference')

        for pref in ['lte', 'umts', 'gsm']:
            self.cellular_page.set_connection_type(pref)
            self.assertThat(get_pref, Eventually(Equals(pref)))

    def test_current_carrier(self):
        self.assertThat(lambda: self.modem_0.Get(NETREG_IFACE, 'Name'),
                        Eventually(Equals('fake.tel')))

    def test_change_carrier(self):
        self.cellular_page.change_carrier('my.cool.telco')

    def test_no_forbidden_network(self):
        """ Ensures that a forbidden network is not shown """
        self.assertThat(
            lambda: self.cellular_page.change_carrier('my.bad.telco'),
            raises(StateNotFoundError)
        )


class DualSimCellularTestCase(CellularBaseTestCase):

    use_sims = 2

    def test_data_off(self):
        self.ctv_private.Set(CON_IFACE,
                             'SimForMobileData',
                             dbus.ObjectPath("/"))
        self.cellular_page.select_sim_for_data('/ril_0')
        self.ctv_private.Set(CON_IFACE, 'MobileDataEnabled', True)
        self.cellular_page.disable_data()
        self.assertThat(
            lambda: self.ctv_private.Get(CON_IFACE, 'MobileDataEnabled'),
            Eventually(Equals(False))
        )

    def test_sim1_online(self):
        self.ctv_private.Set(CON_IFACE,
                             'SimForMobileData',
                             dbus.ObjectPath("/"))
        self.cellular_page.select_sim_for_data('/ril_0')
        self.assertThat(
            lambda: self.ctv_private.Get(CON_IFACE, 'SimForMobileData'),
            Eventually(Equals(self.ctv_sim0.object_path))
        )

    def test_sim2_online(self):
        self.ctv_private.Set(CON_IFACE,
                             'SimForMobileData',
                             dbus.ObjectPath("/"))
        self.cellular_page.select_sim_for_data('/ril_1')
        self.assertThat(
            lambda: self.ctv_private.Get(CON_IFACE, 'SimForMobileData'),
            Eventually(Equals(self.ctv_sim1.object_path))
        )

    def test_connection_type_on_sim1(self):

        def get_pref():
            return self.modem_0.Get(RDO_IFACE, 'TechnologyPreference')

        sim = '/ril_0'
        stac = self.system_settings.main_view.scroll_to_and_click
        self.cellular_page.select_sim_for_data(sim)
        for pref in ['lte', 'umts', 'gsm']:
            self.cellular_page.set_connection_type(
                pref, sim=sim, scroll_to_and_click=stac)
            self.assertThat(get_pref, Eventually(Equals(pref)))

    def test_connection_type_on_sim2(self):

        def get_pref():
            return self.modem_1.Get(RDO_IFACE, 'TechnologyPreference')

        sim = '/ril_1'
        stac = self.system_settings.main_view.scroll_to_and_click
        self.cellular_page.select_sim_for_data(sim)
        for pref in ['gsm']:
            self.cellular_page.set_connection_type(
                pref, sim=sim, scroll_to_and_click=stac)
            self.assertThat(get_pref, Eventually(Equals(pref)))

    def test_current_carrier_sim1(self):
        self.assertThat(lambda: self.modem_0.Get(NETREG_IFACE, 'Name'),
                        Eventually(Equals('fake.tel')))

    def test_change_carrier_sim1(self):
        sim = '/ril_0'
        self.cellular_page.change_carrier('my.cool.telco', sim=sim)

    def test_current_carrier_sim2(self):
        self.assertThat(lambda: self.modem_1.Get(NETREG_IFACE, 'Name'),
                        Eventually(Equals('fake.tel')))

    def test_change_carrier_sim2(self):
        sim = '/ril_1'
        self.cellular_page.change_carrier('my.cool.telco', sim=sim)

    def test_change_sim1_name(self):

        sim = '/ril_0'
        try:
            old_name = self.obj_phone.GetSimNames()[sim]
        except Exception:
            old_name = 'SIM 1'
        new_name = 'FOO BAR'
        self.cellular_page.set_name(sim, new_name)

        try:
            self.assertThat(
                lambda: self.obj_phone.GetSimNames()[sim],
                Eventually(Equals(new_name)))
        except Exception as e:
            raise e
        finally:
            self.cellular_page.set_name(sim, old_name)

    def test_change_sim2_name(self):
        sim = '/ril_1'

        try:
            old_name = self.obj_phone.GetSimNames()[sim]
        except Exception:
            old_name = 'SIM 2'

        new_name = 'BAR BAZ'
        self.cellular_page.set_name(sim, new_name)

        try:
            self.assertThat(
                lambda: self.obj_phone.GetSimNames()[sim],
                Eventually(Equals(new_name)))
        except Exception as e:
            raise e
        finally:
            self.cellular_page.set_name(sim, old_name)

    def test_remote_manipulation_of_name(self):
        old_names = self.obj_phone.GetSimNames()
        sim = '/ril_0'
        name = 'BAS QUX'
        new_names = old_names
        new_names[sim] = name
        self.obj_phone.Set(ACCOUNTS_PHONE_IFACE, "SimNames", new_names)
        self.dbus_mock.EmitSignal(ACCOUNTS_PHONE_IFACE, "PropertyChanged",
                                  "sv", dbus.Array(["SimNames", new_names]))

        try:
            self.assertThat(
                lambda: self.cellular_page.get_name(sim),
                Eventually(StartsWith(name)))
        except Exception as e:
            raise e

    def test_roaming_switch(self):
        self.cellular_page.disable_data()
        # assert roaming_switch is disabled
        self.assertThat(
            lambda: self.cellular_page.enable_roaming(timeout=1),
            raises(AssertionError)
        )

    def test_allow_roaming_sim_1(self):
        self.ctv_private.Set(CON_IFACE, 'MobileDataEnabled', True)
        self.ctv_sim0.Set(CTV_SIM_IFACE, 'DataRoamingEnabled', False)
        self.ctv_private.Set(CON_IFACE,
                             'SimForMobileData',
                             dbus.ObjectPath("/"))
        self.cellular_page.select_sim_for_data('/ril_0')
        self.assertThat(
            lambda: self.ctv_private.Get(CON_IFACE, 'SimForMobileData'),
            Eventually(Equals(self.ctv_sim0.object_path))
        )
        self.cellular_page.enable_roaming()
        self.assertThat(
            lambda: self.ctv_sim0.Get(CTV_SIM_IFACE, 'DataRoamingEnabled'),
            Eventually(Equals(True))
        )

    def test_allow_roaming_sim_2(self):
        self.ctv_private.Set(CON_IFACE, 'MobileDataEnabled', True)
        self.ctv_sim1.Set(CTV_SIM_IFACE, 'DataRoamingEnabled', False)
        self.ctv_private.Set(CON_IFACE,
                             'SimForMobileData',
                             dbus.ObjectPath("/"))
        self.cellular_page.select_sim_for_data('/ril_1')
        self.assertThat(
            lambda: self.ctv_private.Get(CON_IFACE, 'SimForMobileData'),
            Eventually(Equals(self.ctv_sim1.object_path))
        )
        self.cellular_page.enable_roaming()
        self.assertThat(
            lambda: self.ctv_sim1.Get(CTV_SIM_IFACE, 'DataRoamingEnabled'),
            Eventually(Equals(True))
        )

    def test_changing_default_sim_for_calls(self):
        # click ask
        self.system_settings.main_view.scroll_to_and_click(
            self.get_default_sim_for_calls_selector('ask'))
        # click first sim
        self.system_settings.main_view.scroll_to_and_click(
            self.get_default_sim_for_calls_selector('/ril_0'))

        self.assertThat(
            lambda: self.obj_phone.GetDefaultSimForCalls(),
            Eventually(Equals('/ril_0')))

    def test_changing_default_sim_for_messages(self):
        # click ask
        self.system_settings.main_view.scroll_to_and_click(
            self.get_default_sim_for_messages_selector('ask'))
        # click second sim
        self.system_settings.main_view.scroll_to_and_click(
            self.get_default_sim_for_messages_selector('/ril_1'))

        self.assertThat(
            lambda: self.obj_phone.GetDefaultSimForMessages(),
            Eventually(Equals('/ril_1')))


class ApnTestCase(CellularBaseTestCase):

    @skip('Skip until we can get the CPO for ListItem trigger actions')
    def test_remove_apn(self):
        self.add_connection_context(self.modem_0, Type='mms', Name='Failed')
        contexts = self.modem_0.connMan.GetContexts()

        # Assert there's a Failed mms context
        self.assertEqual(1, len(contexts))
        self.assertEqual('/ril_0/context0', contexts[0][0])
        self.assertEqual('Failed', contexts[0][1]['Name'])

        # We delete the failed context
        self.cellular_page.delete_apn('Failed')

    def test_create_internet_apn(self):
        editor = self.cellular_page.open_apn_editor(None)
        editor.set_type('internet')
        editor.set_name('Ubuntu')
        editor.set_access_point_name('ubuntu.ap')
        editor.set_username('user')
        editor.set_password('pass')
        editor.save()

        # Wait for our new context to appear.
        self.assertThat(
            lambda: self.modem_0.connMan.GetContexts()[0][0],
            Eventually(Equals('/ril_0/context0'))
        )
        # Wait for our context to be renamed from the default ofono name.
        self.assertThat(
            lambda: self.modem_0.connMan.GetContexts()[0][1]['Name'],
            Eventually(Equals('Ubuntu'))
        )
        contexts = self.modem_0.connMan.GetContexts()
        new_ctx = contexts[0][1]
        self.assertEqual(1, len(contexts))
        self.assertEqual('internet', new_ctx['Type'])
        self.assertEqual('user', new_ctx['Username'])
        self.assertEqual('pass', new_ctx['Password'])

    def test_create_mms_apn(self):
        editor = self.cellular_page.open_apn_editor(None)
        editor.set_type('mms')
        editor.set_name('Ubuntu')
        editor.set_access_point_name('ubuntu.ap')
        editor.set_message_center('ubuntu.com')
        editor.set_message_proxy('ubuntu:8080')
        editor.set_username('user')
        editor.set_password('pass')
        editor.save()

        # Wait for our new context to appear.
        self.assertThat(
            lambda: self.modem_0.connMan.GetContexts()[0][0],
            Eventually(Equals('/ril_0/context0'))
        )
        # Wait for our context to be renamed from the default ofono name.
        self.assertThat(
            lambda: self.modem_0.connMan.GetContexts()[0][1]['Name'],
            Eventually(Equals('Ubuntu'))
        )
        contexts = self.modem_0.connMan.GetContexts()
        new_ctx = contexts[0][1]
        self.assertEqual(1, len(contexts))
        self.assertEqual('mms', new_ctx['Type'])
        self.assertEqual('ubuntu.ap', new_ctx['AccessPointName'])
        self.assertEqual('http://ubuntu.com', new_ctx['MessageCenter'])
        self.assertEqual('ubuntu:8080', new_ctx['MessageProxy'])
        self.assertEqual('user', new_ctx['Username'])
        self.assertEqual('pass', new_ctx['Password'])

    def test_create_mms_and_internet_apn(self):
        editor = self.cellular_page.open_apn_editor(None)
        editor.set_type('internet+mms')
        editor.set_name('Ubuntu')
        editor.set_access_point_name('ubuntu.ap')
        editor.set_message_center('ubuntu.com')
        editor.set_message_proxy('ubuntu:8080')
        editor.set_username('user')
        editor.set_password('pass')
        editor.save()

        # Wait for our new context to appear.
        self.assertThat(
            lambda: self.modem_0.connMan.GetContexts()[0][0],
            Eventually(Equals('/ril_0/context0'))
        )
        # Wait for our context to be renamed from the default ofono name.
        self.assertThat(
            lambda: self.modem_0.connMan.GetContexts()[0][1]['Name'],
            Eventually(Equals('Ubuntu'))
        )
        contexts = self.modem_0.connMan.GetContexts()
        new_ctx = contexts[0][1]
        self.assertEqual(1, len(contexts))
        self.assertEqual('internet', new_ctx['Type'])
        self.assertEqual('ubuntu.ap', new_ctx['AccessPointName'])
        self.assertEqual('http://ubuntu.com', new_ctx['MessageCenter'])
        self.assertEqual('ubuntu:8080', new_ctx['MessageProxy'])
        self.assertEqual('user', new_ctx['Username'])
        self.assertEqual('pass', new_ctx['Password'])

    def create_lte_apn(self):
        editor = self.cellular_page.open_apn_editor(None)
        editor.set_type('ia')
        editor.set_name('Ubuntu')
        editor.set_access_point_name('ubuntu.ap')
        editor.set_username('user')
        editor.set_password('pass')
        editor.save()

        # Wait for our new context to appear.
        self.assertThat(
            lambda: self.modem_0.connMan.GetContexts()[0][0],
            Eventually(Equals('/ril_0/context0'))
        )
        # Wait for our context to be renamed from the default ofono name.
        self.assertThat(
            lambda: self.modem_0.connMan.GetContexts()[0][1]['Name'],
            Eventually(Equals('Ubuntu'))
        )
        contexts = self.modem_0.connMan.GetContexts()
        new_ctx = contexts[0][1]
        self.assertEqual(1, len(contexts))
        self.assertEqual('ia', new_ctx['Type'])
        self.assertEqual('user', new_ctx['Username'])
        self.assertEqual('pass', new_ctx['Password'])

    def select_apn(self):
        self.add_connection_context(self.modem_0,
                                    Type='internet', Name='Provisioned')

        self.cellular_page.prefer_apn('Provisioned')

        # Assert that Preferred becomes true.
        self.assertThat(
            lambda: self.modem_0.connMan.GetContexts()[0][1]['Preferred'],
            Eventually(Equals(True))
        )

    def active_apn_is_automatically_preferred(self):
        self.add_connection_context(self.modem_0,
                                    Type='internet', Name='Provisioned',
                                    Active=True)
        self.cellular_page.open_apn_page(None)
        self.assertThat(
            lambda: self.modem_0.connMan.GetContexts()[0][1]['Preferred'],
            Eventually(Equals(True))
        )



import argparse
import json
import sys
import threading

from datetime import datetime
from http.server import BaseHTTPRequestHandler, HTTPServer
from urllib.parse import urlparse


def log(msg):
    fd = sys.stdout
    fd.write('%s %s\n' % (datetime.now().strftime('%H:%M:%S'), msg))
    fd.flush()


class Handler(BaseHTTPRequestHandler):

    responses = {}

    def do_HEAD(self):
        """Sends headers.."""
        self.send_response(200)
        self.send_header("X-Click-Token", "Mock-X-Click-Token")
        self.end_headers()

    def do_POST(self):
        """Respond to a POST request."""
        self.do_GET()

    def do_GET(self):
        """Respond to a GET request."""
        r = None
        o = urlparse(self.path)
        for response in self.server.responses:
            if '*' in response['path']:
                r = response
            elif o.path in response['path']:
                r = response

        if not r:
            self.log_message('Could not handle request')
            print(o.path)
            return

        self.send_response(r['status_code'])
        self.send_header("Content-type", 'application/json')
        self.end_headers()

        if 'content' in r:
            try:
                self.wfile.write(
                    json.dumps(r['content']).encode('utf-8')
                )
            except BrokenPipeError:
                # System Settings shut down before we finished
                # up. Log and ignore.
                self.log_message('Server was interrupted.')


class Manager(object):

    # TODO: Use server_port=0
    def __init__(self, server_address='', server_port=9009, responses={},
                 cmdline=False):
        """Creates and initializes a Manager object. If there's an asterisk
        in the responses dict, it's used to handle all paths."""
        self._thread = None
        self._cmdline = cmdline
        if not responses:
            responses = [
                {
                    'path': '/metadata',
                    'status_code': 200,
                    'content_type': 'application/json',
                    'content': [{
                        "name": "com.ubports.dropping-letters",
                        "version": "0.1.2.3",
                        "icon_url": (
                            "https://raw.githubusercontent.com/ninja-ide/"
                            "ninja-ide/master/ninja_ide/img/ninja_icon.png"
                        ),
                        "download_url": ("https://upload.wikimedia.org/wikiped"
                                         "ia/commons/3/30/04U_Dec_28_2012_0230"
                                         "Z.jpg"),
                        "binary_filesize": 23820.0 * 1000.0,
                        "download_sha512": "foo",
                        "changelog": "New Dropping Letters.",
                        "title": "Dropping Letters game"
                    }, {
                        "name": "com.ubports.stock-ticker-mobile",
                        "version": "0.3.7ubuntu2",
                        "icon_url": (
                            "https://upload.wikimedia.org/wikipedia/"
                            "commons/a/ab/Logo-ubuntu_cof-orange-hex.svg"
                        ),
                        "download_url": ("https://upload.wikimedia.org/wikiped"
                                         "ia/commons/d/d2/The_Harvesters%2C_p"
                                         "ainting_by_Brugel%2C_with_frame.jpg"
                                         ),
                        "binary_filesize": 5015.2 * 1000.0,
                        "download_sha512": "bar",
                        "changelog": "New ticker.",
                        "title": "A stock trading app with charts, news, and management"  # noqa
                    }]
                },
                {
                    'path': '/403',
                    'status_code': 403
                },
                {
                    'path': '/404',
                    'status_code': 404
                },
            ]
        self._httpd = HTTPServer((server_address, server_port), Handler)
        self._httpd.responses = responses
        log('Created mock update click server.')

    def is_running(self):
        return self._thread.is_alive()

    def start(self):
        self._thread = threading.Thread(target=self._httpd.serve_forever)
        self._thread.start()
        log(
            'Started mock update click server on http://%s:%d.' % (
                self._httpd.server_address
            )
        )

        # If the command line is the caller, wait for the keyboard interrupt.
        # TODO: infer this by checking sys?
        if self._cmdline:
            print('Ctrl-C stops this server.')
            try:
                while self.is_running():
                    self._thread.join(5)
            except (KeyboardInterrupt, SystemExit):
                print('')
                self.stop()

    def stop(self):
        self._httpd.shutdown()
        self._httpd.server_close()
        self._thread.join(timeout=10.0)
        if self.is_running():
            raise 'Failed to stop server'
        log('Stopped mock update click server.')


def parse_args():
    parser = argparse.ArgumentParser(description='mock click update server')

    parser.add_argument('-a', '--address', default='localhost',
                        help='address of server (default: localhost)')
    parser.add_argument('-p', '--port', type=int, default=9009,
                        help='port of server (default: 9009)')

    parser.add_argument('-r', '--responses',
                        help='JSON dictionary of path to responses. '
                             'Passing a * (asteriks) path will used '
                             'in all responses.')

    args = parser.parse_args()

    return args


if __name__ == '__main__':
    args = parse_args()

    responses = None
    if args.responses:
        try:
            responses = json.loads(args.responses)
        except ValueError as detail:
            sys.stderr.write('Malformed JSON given for '
                             'responses: %s\n' % detail)
            sys.exit(2)

        if not isinstance(responses, list):
            sys.stderr.write('JSON responses must be a list\n')
            sys.exit(2)

    man = Manager(
        server_address=args.address,
        server_port=args.port,
        responses=responses,
        cmdline=True)

    man.start()



#!/usr/bin/python3

# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2013, 2014 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.


import setuptools


setuptools.setup(
    name='lomiri-system-settings',
    version='1.0.1',
    description='Lomiri System Settings autopilot tests.',
    url='https://gitlab.com/ubports/core/lomiri-system-settings',
    license='GPLv3',
    packages=setuptools.find_packages(),
    package_dir={
        'lomiri_system_settings': './lomiri_system_settings'},
    package_data={
        'lomiri_system_settings': ['background_images/*.jpg'],
    }
)



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2014 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.

from time import sleep
from autopilot.matchers import Eventually
from testtools.matchers import Equals, NotEquals

from lomiri_system_settings.tests import SoundBaseTestCase
from lomiri_system_settings.utils.i18n import ugettext as _


class SoundTestCase(SoundBaseTestCase):
    """ Tests for Sound Page """

    def test_sound_page(self):
        """ Checks whether Sound page is available """
        self.assertThat(
            self.sound_page,
            NotEquals(None)
        )
        self.assertThat(
            self.sound_page.title,
            Equals(_('Sound'))
        )

    def test_silent_mode_sound_switch(self):
        """ Check that silent_mode is present and clickable"""
        snd = self.sound_page.select_single(
            objectName="silentMode"
        )
        self.main_view.scroll_to_and_click(snd)
        sleep(0.2)
        calls = self.dbus_mock_isound.GetCalls()
        self.assertThat(
            calls[0][1],
            Equals('Activate')
        )
        self.assertThat(
            calls[0][2][0],
            Equals('silent-mode')
        )

    def test_call_vibrate_sound_switch(self):
        """ Check that call vibrate is present and clickable"""
        snd = self.sound_page.select_single(
            objectName="callVibrate"
        )
        prev_value = self.obj_snd.GetIncomingCallVibrate()
        self.main_view.scroll_to_and_click(snd)
        sleep(0.2)
        self.assertNotEqual(
            self.obj_snd.GetIncomingCallVibrate(),
            prev_value)

    def test_message_vibrate_sound_switch(self):
        """ Check that message vibrate is present and clickable"""
        snd = self.sound_page.select_single(
            objectName="messageVibrate"
        )
        prev_value = self.obj_snd.GetIncomingMessageVibrate()
        self.main_view.scroll_to_and_click(snd)
        sleep(0.2)
        self.assertNotEqual(
            self.obj_snd.GetIncomingMessageVibrate(),
            prev_value)

    def test_message_vibrate_silent_mode_sound_switch(self):
        """ Check that message vibrate silent mode is present and clickable"""
        snd = self.sound_page.select_single(
            objectName="messageVibrateSilentMode"
        )
        prev_value = self.obj_snd.GetIncomingMessageVibrateSilentMode()
        self.main_view.scroll_to_and_click(snd)
        sleep(0.2)
        self.assertNotEqual(
            self.obj_snd.GetIncomingMessageVibrateSilentMode(),
            prev_value)

    def test_keyboard_sound_switch(self):
        """ Check that keyboard sound is present and clickable"""
        kbd_snd = self.sound_page.select_single(
            objectName="keyboardSoundSwitch"
        )
        current_value = kbd_snd.get_properties()["checked"]
        self.main_view.scroll_to_and_click(kbd_snd)
        self.assertThat(
            kbd_snd.get_properties()["checked"], NotEquals(current_value))

    def test_dialpad_sounds_switch(self):
        snd = self.sound_page.select_single(
            objectName="dialpadSounds"
        )
        prev_value = self.obj_snd.GetDialpadSoundsEnabled()
        self.main_view.scroll_to_and_click(snd)
        self.assertThat(
            lambda: self.obj_snd.GetDialpadSoundsEnabled(),
            Eventually(NotEquals(prev_value)))




# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2014-2016 Canonical Ltd.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

from autopilot import introspection
from autopilot.exceptions import StateNotFoundError
from lomiri_system_settings.utils.i18n import ugettext as _

import logging
from time import sleep
import autopilot.logging
import lomiriuitoolkit
import lomiri_system_settings.utils as utils

# TODO This is a workaround for bug #1327325 that will make phabet-test-run
# fail if something is printed to stdout.
logging.basicConfig(filename='warning.log', level=logging.WARNING)
logger = logging.getLogger(__name__)


class SystemSettings(lomiriuitoolkit.LomiriUIToolkitCustomProxyObjectBase):

    """Helper class for System Settings application"""

    @classmethod
    def validate_dbus_object(cls, path, state):
        name = introspection.get_classname_from_path(path)
        return ((name == b'SystemSettings' and
                 state['applicationName'][1] == 'SystemSettings') or
                (name == b'lomiri-system-settings' and
                 state['applicationName'][1] == 'lomiri-system-settings'))

    @property
    def main_view(self):
        """Return main view"""
        return self.select_single(objectName='systemSettingsMainView')


class SystemSettingsMainWindow(lomiriuitoolkit.MainView):

    """Autopilot helper for the Main Window."""

    @classmethod
    def validate_dbus_object(cls, path, state):
        name = introspection.get_classname_from_path(path)
        return (name == b'MainWindow' and
                state['objectName'][1] == 'systemSettingsMainView')

    @autopilot.logging.log_action(logger.debug)
    def click_item(self, object_name):
        """Click a system settings item.

        :param object_name: The objectName property of the item to click.

        """
        item = self.select_single(objectName=object_name)
        item.swipe_into_view()
        self.pointing_device.click_object(item)

    @autopilot.logging.log_action(logger.debug)
    def go_to_reset_phone(self):
        return self._go_to_page('entryComponent-reset', 'resetPage')

    @autopilot.logging.log_action(logger.debug)
    def go_to_language_page(self):
        return self._go_to_page('entryComponent-language', 'languagePage')

    @autopilot.logging.log_action(logger.debug)
    def go_to_wifi_page(self):
        return self._go_to_page('entryComponent-wifi', 'wifiPage')

    @autopilot.logging.log_action(logger.debug)
    def go_to_cellular_page(self):
        return self._go_to_page('entryComponent-cellular', 'cellularPage')

    @autopilot.logging.log_action(logger.debug)
    def go_to_hotspot_page(self):
        return self._go_to_page('entryComponent-hotspot', 'hotspotPage')

    @autopilot.logging.log_action(logger.debug)
    def go_to_bluetooth_page(self):
        return self._go_to_page('entryComponent-bluetooth', 'bluetoothPage')

    @autopilot.logging.log_action(logger.debug)
    def go_to_phone_page(self):
        return self._go_to_page('entryComponent-phone', 'phonePage')

    @autopilot.logging.log_action(logger.debug)
    def go_to_about_page(self):
        return self._go_to_page('entryComponent-about', 'aboutPage')

    @autopilot.logging.log_action(logger.debug)
    def go_to_sound_page(self):
        return self._go_to_page('entryComponent-sound', 'soundPage')

    @autopilot.logging.log_action(logger.debug)
    def go_to_security_page(self):
        return self._go_to_page('entryComponent-security-privacy',
                                'securityPrivacyPage')

    @autopilot.logging.log_action(logger.debug)
    def go_to_notification_page(self):
        return self._go_to_page('entryComponent-notifications',
                                'systemNotificationsPage')

    @autopilot.logging.log_action(logger.debug)
    def go_to_datetime_page(self):
        return self._go_to_page('entryComponent-time-date', 'timeDatePage')

    @autopilot.logging.log_action(logger.debug)
    def go_to_vpn_page(self):
        return self._go_to_page('entryComponent-vpn', 'vpnPage')

    def _go_to_page(self, item_object_name, page_object_name):
        self.click_item(item_object_name)
        page = self.wait_select_single(objectName=page_object_name)
        page.active.wait_for(True)
        return page

    @autopilot.logging.log_action(logger.debug)
    def scroll_to(self, obj):

        def get_page_bottom():
            return page.globalRect[1] + page.globalRect[3]

        page = self.system_settings_page
        page_right = page.globalRect[0] + page.globalRect[2]
        page_bottom = get_page_bottom()
        page_center_x = int(page_right / 2)
        page_center_y = int(page_bottom / 2)
        while obj.globalRect[1] + obj.height > get_page_bottom():
            self.pointing_device.drag(
                page_center_x,
                page_center_y,
                page_center_x,
                page_center_y - obj.height * 2
            )
            # avoid a flick, this is very important
            sleep(0.3)

    def scroll_to_and_click(self, obj):
        self.scroll_to(obj)
        self.pointing_device.click_object(obj)

    @property
    def system_settings_page(self):
        return self.select_single(objectName='systemSettingsPage')

    @property
    def choose_page(self):
        """ Return 'Choose carrier' page """
        return self.select_single(objectName="chooseCarrierPage")

    @property
    def storage_page(self):
        """ Return 'Storage' page """
        return self.select_single(objectName='storagePage')

    @property
    def updates_page(self):
        """ Return 'System Update' page """
        return self.select_single(objectName='systemUpdatesPage')

    @property
    def background_page(self):
        """ Return 'Background' page """
        return self.select_single(objectName='backgroundPage')

    @property
    def _orientation_lock_switch(self):
        return self.wait_select_single(
            lomiriuitoolkit.CheckBox,
            objectName='orientationLockSwitch')

    def enable_orientation_lock(self):
        self._orientation_lock_switch.check()

    def disable_orientation_lock(self):
        self._orientation_lock_switch.uncheck()


class Dialog(lomiriuitoolkit.Dialog):
    # XXX A new Dialog custom proxy object was added to the toolkit.
    # Because of https://bugs.launchpad.net/autopilot-qt/+bug/1341671
    # we need to make sure it does not match in any selection.

    @classmethod
    def validate_dbus_object(cls, path, state):
        return False


class LabelTextField(lomiriuitoolkit.TextField):
    """LabelTextField is a component local to the APN Editor in the cellular
    plugin."""
    pass


class CellularPage(lomiriuitoolkit.LomiriUIToolkitCustomProxyObjectBase):

    """Autopilot helper for the Cellular page."""

    @classmethod
    def validate_dbus_object(cls, path, state):
        name = introspection.get_classname_from_path(path)
        if name == b'PageComponent':
            if state['objectName'][1] == 'cellularPage':
                return True
        return False

    @autopilot.logging.log_action(logger.debug)
    def enable_data(self):
        self._set_data(True)

    @autopilot.logging.log_action(logger.debug)
    def disable_data(self):
        self._set_data(False)

    @autopilot.logging.log_action(logger.debug)
    def _set_data(self, data):
        chk = self.select_single(objectName='data')
        if data:
            chk.check()
        else:
            chk.uncheck()

    @autopilot.logging.log_action(logger.debug)
    def get_data(self):
        return self.select_single(objectName='data').checked

    @autopilot.logging.log_action(logger.debug)
    def enable_roaming(self, timeout=10):
        self._set_roaming(True, timeout=timeout)

    @autopilot.logging.log_action(logger.debug)
    def disable_roaming(self, timeout=10):
        self._set_roaming(False, timeout=timeout)

    @autopilot.logging.log_action(logger.debug)
    def _set_roaming(self, roaming, timeout):
        chk = self.select_single(objectName='roaming')
        if roaming:
            chk.check(timeout=timeout)
        else:
            chk.uncheck(timeout=timeout)

    @autopilot.logging.log_action(logger.debug)
    def set_connection_type(self, radio_type, sim='/ril_0',
                            scroll_to_and_click=None):
        self._set_connection_type(radio_type, sim, scroll_to_and_click)

    @autopilot.logging.log_action(logger.debug)
    def _set_connection_type(self, radio_type, sim, scroll_to_and_click):
        t = self.wait_select_single(
            'OptionSelectorDelegate',
            objectName='%s_radio_%s' % (sim, radio_type))
        if scroll_to_and_click:
            scroll_to_and_click(t)
        else:
            t.swipe_into_view()

        self.pointing_device.click_object(t)

    @autopilot.logging.log_action(logger.debug)
    def change_carrier(self, carrier, sim=None):
        carrierApnPage = self._click_carrier_apn()
        chooseCarrierPage = carrierApnPage.open_carrier(sim)
        chooseCarrierPage.set_carrier(carrier)

    @autopilot.logging.log_action(logger.debug)
    def open_apn_editor(self, name, sim=None):
        carrierApnPage = self._click_carrier_apn()
        chooseApnPage = carrierApnPage.open_apn(sim)
        return chooseApnPage.open(name)

    @autopilot.logging.log_action(logger.debug)
    def open_apn_page(self, name, sim=None):
        carrierApnPage = self._click_carrier_apn()
        chooseApnPage = carrierApnPage.open_apn(sim)
        return chooseApnPage

    @autopilot.logging.log_action(logger.debug)
    def delete_apn(self, name, sim=None):
        carrierApnPage = self._click_carrier_apn()
        chooseApnPage = carrierApnPage.open_apn(sim)
        return chooseApnPage.delete(name)

    @autopilot.logging.log_action(logger.debug)
    def prefer_apn(self, name, sim=None):
        carrierApnPage = self._click_carrier_apn()
        chooseApnPage = carrierApnPage.open_apn(sim)
        return chooseApnPage.check(name)

    @autopilot.logging.log_action(logger.debug)
    def _click_carrier_apn(self):
        item = self.select_single(objectName='carrierApnEntry')
        self.pointing_device.click_object(item)
        return self.get_root_instance().wait_select_single(
            objectName='carrierApnPage')

    @autopilot.logging.log_action(logger.debug)
    def select_sim_for_data(self, sim):
        self._select_sim_for_data(sim)

    @autopilot.logging.log_action(logger.debug)
    def _select_sim_for_data(self, sim):
        item = self.select_single(objectName='use%s' % sim)
        self.pointing_device.click_object(item)

    @autopilot.logging.log_action(logger.debug)
    def select_sim_for_calls(self, sim):
        pass

    @autopilot.logging.log_action(logger.debug)
    def select_sim_for_messages(self):
        pass

    @autopilot.logging.log_action(logger.debug)
    def set_name(self, sim, name):
        self._set_name(sim, name)

    def get_name(self, sim):
        obj = self.select_single(
            objectName="label_%s" % sim)
        return obj.text

    def _set_name(self, sim, name):
        obj = self.select_single(
            objectName="edit_name_%s" % sim)
        self.pointing_device.click_object(obj)

        # wait for animation
        sleep(1)
        ok = self.select_single('Button', objectName="doRename")

        field = self.wait_select_single('TextField', objectName="nameField")
        field.write(name)
        self.pointing_device.click_object(ok)


class HotspotPage(lomiriuitoolkit.LomiriUIToolkitCustomProxyObjectBase):

    """Autopilot helper for Hotspot page."""

    @classmethod
    def validate_dbus_object(cls, path, state):
        name = introspection.get_classname_from_path(path)
        if name == b'ItemPage':
            if state['objectName'][1] == 'hotspotPage':
                return True
        return False

    @property
    def _switch(self):
        return self.wait_select_single(
            lomiriuitoolkit.CheckBox,
            objectName='hotspotSwitch')

    @autopilot.logging.log_action(logger.debug)
    def enable_hotspot(self):
        # We assume that the following AssertionError is due to the panel
        # instantly setting checked to False, prompting the user to turn on
        # Wi-Fi instead.
        try:
            self._switch.check(timeout=2)
        except AssertionError:
            pass

        try:
            prompt = self.get_root_instance().wait_select_single(
                objectName='enableWifiDialog')
        except StateNotFoundError:
            prompt = None

        if prompt:
            prompt.confirm_enable()
            prompt.wait_until_destroyed(timeout=5)

    @autopilot.logging.log_action(logger.debug)
    def disable_hotspot(self):
        self._switch.uncheck()

    @autopilot.logging.log_action(logger.debug)
    def setup_hotspot(self, config):
        obj = self.select_single(objectName='hotspotSetupButton')
        self.pointing_device.click_object(obj)
        setup = self.get_root_instance().wait_select_single(
            objectName='hotspotSetup')
        if config:
            if 'ssid' in config:
                setup.set_ssid(config['ssid'])
            if 'auth' in config:
                setup.set_auth(config['auth'])
            if 'password' in config:
                setup.set_password(config['password'])
        utils.dismiss_osk()
        setup.enable()
        if setup:
            setup.wait_until_destroyed()

    @autopilot.logging.log_action(logger.debug)
    def get_hotspot_status(self):
        return self._switch.checked

    @autopilot.logging.log_action(logger.debug)
    def get_hotspot_possible(self):
        return self._switch.enabled


class HotspotEnableWifiDialog(
        lomiriuitoolkit.LomiriUIToolkitCustomProxyObjectBase):
    """Autopilot helper for the 'Turn on Wi-Fi' dialog in hotspot panel."""

    @classmethod
    def validate_dbus_object(cls, path, state):
        name = introspection.get_classname_from_path(path)
        if name == b'Dialog':
            if state['objectName'][1] == 'enableWifiDialog':
                return True
        return False

    @autopilot.logging.log_action(logger.debug)
    def confirm_enable(self):
        button = self.select_single('Button', objectName='confirmEnable')
        self.pointing_device.click_object(button)


class HotspotSetup(lomiriuitoolkit.LomiriUIToolkitCustomProxyObjectBase):

    """Autopilot helper for Hotspot setup."""

    @classmethod
    def validate_dbus_object(cls, path, state):
        name = introspection.get_classname_from_path(path)
        if name == b'Dialog':
            if state['objectName'][1] == 'hotspotSetup':
                return True
        return False

    @property
    def _ssid_field(self):
        return self.wait_select_single(
            lomiriuitoolkit.TextField,
            objectName='ssidField')

    @property
    def _password_field(self):
        return self.wait_select_single(
            lomiriuitoolkit.TextField,
            objectName='passwordField')

    @property
    def _enable_button(self):
        return self.wait_select_single(
            'Button', objectName='confirmButton')

    @property
    def _password_required_check(self):
        return self.wait_select_single(
            'CheckBox', objectName='passwordRequiredToggle')

    @autopilot.logging.log_action(logger.debug)
    def set_ssid(self, ssid):
        self._ssid_field.write(ssid)

    @autopilot.logging.log_action(logger.debug)
    def set_password(self, password):
        self._password_field.write(password)

    @autopilot.logging.log_action(logger.debug)
    def set_auth(self, auth):
        if auth == 'wpa-psk':
            self._password_required_check.check()
        else:
            self._password_required_check.uncheck()

    @autopilot.logging.log_action(logger.debug)
    def enable(self):
        self.pointing_device.click_object(self._enable_button)


class BluetoothPage(lomiriuitoolkit.LomiriUIToolkitCustomProxyObjectBase):

    """Autopilot helper for Bluetooth page."""

    @classmethod
    def validate_dbus_object(cls, path, state):
        name = introspection.get_classname_from_path(path)
        if name == b'PageComponent':
            if state['objectName'][1] == 'bluetoothPage':
                return True
        return False

    def get_disconnected_devices(self):
        """Return the list of known disconnected devices.

        :return: a list containing the text for each device

        """
        disconnected_list = self.select_single(
            'QQuickColumn',
            objectName='disconnectedList'
        )
        # NOTE: the UI design uses ellipsis to be suggestive
        ellipsis = '\u2026'
        return [device.text.strip(ellipsis) for device in
                disconnected_list.select_many('LabelVisual')]


class PageCarrierAndApn(lomiriuitoolkit.LomiriUIToolkitCustomProxyObjectBase):

    """Autopilot helper for carrier/apn entry page (singlesim)."""
    @autopilot.logging.log_action(logger.debug)
    def open_carrier(self, sim):
        return self._click_carrier(sim)

    @autopilot.logging.log_action(logger.debug)
    def _click_carrier(self, sim):
        obj = self.select_single(
            objectName='carrier')
        self.pointing_device.click_object(obj)
        return self.get_root_instance().wait_select_single(
            objectName='chooseCarrierPage')

    @autopilot.logging.log_action(logger.debug)
    def open_apn(self, sim):
        return self._click_apn(sim)

    @autopilot.logging.log_action(logger.debug)
    def _click_apn(self, sim):
        obj = self.select_single(
            objectName='apn')
        self.pointing_device.click_object(obj)
        return self.get_root_instance().wait_select_single(
            objectName='apnPage')


class PageCarriersAndApns(
        lomiriuitoolkit.LomiriUIToolkitCustomProxyObjectBase):
    """Autopilot helper for carrier/apn entry page (multisim)."""
    @autopilot.logging.log_action(logger.debug)
    def open_carrier(self, sim):
        return self._click_carrier(sim)

    @autopilot.logging.log_action(logger.debug)
    def _click_carrier(self, sim):
        obj = self.select_single(
            objectName='%s_carriers' % sim)
        self.pointing_device.click_object(obj)
        return self.get_root_instance().wait_select_single(
            objectName='chooseCarrierPage')


class PageChooseCarrier(lomiriuitoolkit.LomiriUIToolkitCustomProxyObjectBase):

    """Autopilot helper for carrier selection page"""

    @autopilot.logging.log_action(logger.debug)
    def set_automatic(self):
        item = self.select_single(text='Automatically')
        self.pointing_device.click_object(item)

    @autopilot.logging.log_action(logger.debug)
    def set_carrier(self, carrier):
        # wait for animation, since page.animationRunning.wait_for(False)
        # does not work?
        sleep(0.5)
        allOperators = self.select_single(objectName="allOperators")
        otherOperators = self.select_single(objectName="otherOperators")

        if allOperators.visible:
            opList = allOperators
        elif otherOperators.visible:
            opList = otherOperators
        else:
            raise Exception("No operator list visible.")

        item = opList.select_single(text=carrier, objectName="carrier")
        self.pointing_device.click_object(item)


class PageChooseApn(lomiriuitoolkit.LomiriUIToolkitCustomProxyObjectBase):

    """Autopilot helper for apn editor page"""

    @autopilot.logging.log_action(logger.debug)
    def open(self, name):
        return self._open_editor(name)

    @autopilot.logging.log_action(logger.debug)
    def delete(self, name):
        self._delete(name)

    @autopilot.logging.log_action(logger.debug)
    def _delete(self, name):
        item = self.wait_select_single('Standard', objectName='edit_%s' % name)
        item.swipe_to_delete()
        item.confirm_removal()

    @autopilot.logging.log_action(logger.debug)
    def check(self, name):
        self._check(name)

    @autopilot.logging.log_action(logger.debug)
    def _check(self, name):
        item = self.wait_select_single(
            'CheckBox', objectName='%s_preferred' % name
        )
        item.check()

    @autopilot.logging.log_action(logger.debug)
    def _open_editor(self, name):
        if name:
            item = self.select_single(objectName='edit_%s' % name)
            self.pointing_device.click_object(item)
        else:
            main_view = self.get_root_instance().select_single(
                objectName='systemSettingsMainView')
            header = main_view.select_single('AppHeader')
            header.click_action_button('newApn')
        return self.get_root_instance().wait_select_single(
            objectName='apnEditor')


class PageApnEditor(lomiriuitoolkit.LomiriUIToolkitCustomProxyObjectBase):

    """Autopilot helper for apn editor page"""

    flickable = None

    def __init__(self, *args):
        super().__init__(*args)
        self.flickable = self.select_single(objectName='scrollArea')

    @autopilot.logging.log_action(logger.debug)
    def set_type(self, t):
        selector = self.select_single(
            'ItemSelector', objectName='typeSelector')
        self.pointing_device.click_object(selector)
        selector.currentlyExpanded.wait_for(True)
        item = self.select_single(objectName='type_%s' % t)

        # A bit dirty
        while selector.currentlyExpanded:
            self.pointing_device.click_object(item)

    @autopilot.logging.log_action(logger.debug)
    def set_name(self, new_name):
        self._populate_field('name', new_name)

    @autopilot.logging.log_action(logger.debug)
    def set_access_point_name(self, new_name):
        self._populate_field('accessPointName', new_name)

    @autopilot.logging.log_action(logger.debug)
    def set_message_center(self, new_message_center):
        self._populate_field('messageCenter', new_message_center)

    @autopilot.logging.log_action(logger.debug)
    def set_message_proxy(self, new_message_proxy):
        self._populate_field('messageProxy', new_message_proxy)

        # Sleep for the duration of the timer that will copy any
        # port into the port field
        sleep(1.5)

    @autopilot.logging.log_action(logger.debug)
    def set_port(self, new_port):
        self._populate_field('port', new_port)

    @autopilot.logging.log_action(logger.debug)
    def set_username(self, new_username):
        self._populate_field('username', new_username)

    @autopilot.logging.log_action(logger.debug)
    def set_password(self, new_password):
        self._populate_field('password', new_password)

    def _populate_field(self, field, text):
        f = self.select_single(LabelTextField, objectName=field)
        self.flickable.swipe_child_into_view(f)
        f.write(text)

    @autopilot.logging.log_action(logger.debug)
    def save(self):
        main_view = self.get_root_instance().select_single(
            objectName='systemSettingsMainView')
        header = main_view.select_single('AppHeader')
        header.click_action_button('saveApn')


class SecurityPage(lomiriuitoolkit.QQuickFlickable):

    """Autopilot helper for the Security page."""

    @classmethod
    def validate_dbus_object(cls, path, state):
        name = introspection.get_classname_from_path(path)
        if name == b'PageComponent':
            if state['objectName'][1] == 'securityPrivacyPage':
                return True
        return False

    @autopilot.logging.log_action(logger.debug)
    def go_to_sim_lock(self):
        selector = self.select_single(objectName='simControl')
        self.swipe_child_into_view(selector)
        self.pointing_device.click_object(selector)
        return self.get_root_instance().wait_select_single(
            objectName='simPinPage'
        )


class SimPin(lomiriuitoolkit.QQuickFlickable):

    """Autopilot helper for the SimPin Page."""

    def get_sim_pin_switch(self, sim_number):
        """Return the SIM PIN switch."""
        switches = self.select_many(objectName='simPinSwitch')
        switches = sorted(switches, key=lambda switch: (switch.globalRect.y))
        return switches[sim_number]

    @autopilot.logging.log_action(logger.debug)
    def click_sim_pin_switch(self, sim_number):
        """Click on the SIM PIN switch, return the SIM PIN dialog."""
        sim_pin_switch = self.get_sim_pin_switch(sim_number)
        self.pointing_device.click_object(sim_pin_switch)
        return self.get_root_instance().wait_select_single(
            objectName='lockDialogComponent'
        )

    @autopilot.logging.log_action(logger.debug)
    def enter_lock_pin(self, pin):
        """Enter the given pin into our dialog."""
        root_instance = self.get_root_instance()
        prev_input = root_instance.wait_select_single(
            objectName='prevInput'
        )
        prev_input.write(pin)
        lock_unlock_button = root_instance.select_single(
            objectName='lockButton'
        )
        self.pointing_device.click_object(lock_unlock_button)


class TimeAndDatePage(lomiriuitoolkit.LomiriUIToolkitCustomProxyObjectBase):

    """Autopilot helper for the TimeAndDate page."""

    @classmethod
    def validate_dbus_object(cls, path, state):
        name = introspection.get_classname_from_path(path)
        if name == b'PageComponent':
            if state['objectName'][1] == 'timeDatePage':
                return True
        return False


class SoundPage(lomiriuitoolkit.LomiriUIToolkitCustomProxyObjectBase):

    """Autopilot helper for the Sound page."""

    @classmethod
    def validate_dbus_object(cls, path, state):
        name = introspection.get_classname_from_path(path)
        if name == b'ItemPage':
            if state['objectName'][1] == 'soundPage':
                return True
        return False


class AboutPage(lomiriuitoolkit.LomiriUIToolkitCustomProxyObjectBase):

    """Autopilot helper for the About page."""

    @classmethod
    def validate_dbus_object(cls, path, state):
        name = introspection.get_classname_from_path(path)
        if name == b'PageComponent':
            if state['objectName'][1] == 'aboutPage':
                return True
        return False

    def get_device_name(self):
        device_label = self.select_single(objectName='deviceLabel')
        return device_label.text

    def is_serial_visible(self):
        serial_item = self._get_serial_item()
        return serial_item.visible

    def _get_serial_item(self):
        return self.select_single(objectName='serialItem')

    def get_serial(self):
        serial_item = self._get_serial_item()
        return serial_item.value

    def is_imei_visible(self):
        imei_item = self._get_imei_item()
        return imei_item.visible

    def _get_imei_item(self):
        return self.wait_select_single(objectName='imeiItem')

    def get_imei(self):
        imei_item = self._get_imei_item()
        return imei_item.value

    def get_os_information(self):
        os_item = self.select_single(objectName='osItem')
        return os_item.value

    def get_last_updated_date(self):
        last_updated_item = self.select_single(objectName='lastUpdatedItem')
        return last_updated_item.value

    @autopilot.logging.log_action(logger.info)
    def go_to_check_for_updates(self):
        check_for_updates_button = self.select_single(
            objectName='updateButton')
        check_for_updates_button.swipe_into_view()
        self.pointing_device.click_object(check_for_updates_button)
        system_updates_page = self.get_root_instance().wait_select_single(
            objectName='systemUpdatesPage')
        system_updates_page.active.wait_for(True)
        return system_updates_page

    @autopilot.logging.log_action(logger.info)
    def go_to_software_licenses(self):
        license_item = self.select_single(
            'StandardProgression', objectName='licenseItem')
        license_item.swipe_into_view()
        self.pointing_device.click_object(license_item)
        licenses_page = self.get_root_instance().wait_select_single(
            objectName='licensesPage')
        licenses_page.active.wait_for(True)
        return licenses_page

    def get_number(self, obj):
        number = self.select_single(objectName=obj)
        return number.value


class LicensesPage(lomiriuitoolkit.LomiriUIToolkitCustomProxyObjectBase):

    """Autopilot helper for the Licenses page."""

    @classmethod
    def validate_dbus_object(cls, path, state):
        name = introspection.get_classname_from_path(path)
        if name == b'ItemPage':
            if state['objectName'][1] == 'licensesPage':
                return True
        return False


class SystemUpdatesPage(lomiriuitoolkit.LomiriUIToolkitCustomProxyObjectBase):

    """Autopilot helper for the System Updates page."""

    @classmethod
    def validate_dbus_object(cls, path, state):
        name = introspection.get_classname_from_path(path)
        if name == b'PageComponent':
            if state['objectName'][1] == 'systemUpdatesPage':
                return True
        return False

    def stop_check(self):
        btn = self.select_single(objectName='updatesGlobalStopButton')
        self.pointing_device.click_object(btn)


class PhonePage(lomiriuitoolkit.LomiriUIToolkitCustomProxyObjectBase):

    """Autopilot helper for the Phone page."""

    @classmethod
    def validate_dbus_object(cls, path, state):
        name = introspection.get_classname_from_path(path)
        if name == b'PageComponent':
            if state['objectName'][1] == 'phonePage':
                return True
        return False

    def _go_to_page(self, item_object_name, page_object_name):
        self._click_item(item_object_name)
        page = self.get_root_instance().wait_select_single(
            objectName=page_object_name)
        page.active.wait_for(True)
        return page

    def _click_item(self, object_name):
        item = self.wait_select_single(objectName=object_name)
        item.swipe_into_view()
        self.pointing_device.click_object(item)

    @autopilot.logging.log_action(logger.info)
    def go_to_sim_services(self, sim=None):
        """Open the SIM Services settings page.

        :param sim: Number of what SIM to use, either 1 or 2.
            Required parameter in dual SIM setups
        :returns: The SIM Services settings page.

        """
        find = "simServices"
        if sim is not None:
            find = "simServicesSim%d" % sim

        return self._go_to_page(find, 'servicesPage')

    def get_sim_services_enabled(self, sim=None):
        """Return whether or not Sim Services is enabled.

        :param sim: Number of what SIM to use, either 1 or 2.
            Required parameter in dual SIM setups
        :returns: Whether or not Sim Services is enabled.

        """
        find = "simServices"
        if sim is not None:
            find = "simServicesSim%d" % sim
        return self.select_single(objectName=find).enabled

    @property
    def _dialpad_sounds(self):
        """The dialpad sounds switch."""
        return self.wait_select_single(
            lomiriuitoolkit.CheckBox,
            objectName='dialpadSounds')

    @autopilot.logging.log_action(logger.info)
    def enable_dialpad_sounds(self):
        """Enable dialpad sounds."""
        self._dialpad_sounds.check()

    @autopilot.logging.log_action(logger.info)
    def disable_dialpad_sounds(self):
        """Disable dialpad sounds."""
        self._dialpad_sounds.uncheck()

    @autopilot.logging.log_action(logger.info)
    def _enter_call_waiting(self, sim=None):
        """Open the Call Waiting settings page.

        :param sim: Number of what SIM to use, either 1 or 2.
            Required parameter in dual SIM setups
        :returns: The Call Waiting settings page.

        """
        find = "callWait"
        if sim is not None:
            find = "callWaitSim%d" % sim
        return self._go_to_page(find, 'callWaitingPage')

    @autopilot.logging.log_action(logger.info)
    def enable_call_waiting(self, sim):
        self._enter_call_waiting(sim).enable_call_waiting()

    @autopilot.logging.log_action(logger.info)
    def disable_call_waiting(self, sim):
        self._enter_call_waiting(sim).disable_call_waiting()

    @autopilot.logging.log_action(logger.info)
    def _enter_call_forwarding(self, sim=None):
        """Open the Call Forwarding settings page.

        :param sim: Number of what SIM to use, either 1 or 2.
            Required parameter in dual SIM setups
        :returns: The Call Forwarding settings page.

        """
        find = "callFwd"
        if sim is not None:
            find = "callFwdSim%d" % sim

        return self._go_to_page(find, 'callForwardingPage')

    @autopilot.logging.log_action(logger.info)
    def set_forward_unconditionally(self, number, sim=None):
        """Sets forwarding unconditionally to number on a sim.

        :param sim: Number of what SIM to use, either 1 or 2.
            Required parameter in dual SIM setups
        :param number: Number to which we want to forward.
        :returns: The Call Forwarding settings page.
        """
        fwd_page = self._enter_call_forwarding(sim)
        fwd_page.set_forward_unconditionally(number)
        return fwd_page

    @autopilot.logging.log_action(logger.info)
    def unset_forward_unconditionally(self, sim=None):
        """Disables forwarding unconditionally on a sim.

        :param sim: Number of what SIM to use, either 1 or 2.
            Required parameter in dual SIM setups
        :returns: The Call Forwarding settings page.
        """
        fwd_page = self._enter_call_forwarding(sim)
        fwd_page.unset_forward_unconditionally()
        return fwd_page

    @autopilot.logging.log_action(logger.info)
    def get_forward_unconditionally(self, sim=None):
        """Return forwarding unconditionally value on sim.

        :param sim: Number of what SIM to use, either 1 or 2.
            Required parameter in dual SIM setups
        :returns: The forward unconditionally value.
        """
        fwd_page = self._enter_call_forwarding(sim)
        return fwd_page.get_forward_unconditionally()

    @autopilot.logging.log_action(logger.info)
    def set_forward_on_busy(self, number, sim=None):
        """Sets forwarding when busy to number on a sim.

        :param sim: Number of what SIM to use, either 1 or 2.
            Required parameter in dual SIM setups
        :param number: Number to which we want to forward.
        :returns: The Call Forwarding settings page.
        """
        fwd_page = self._enter_call_forwarding(sim)
        fwd_page.set_forward_on_busy(number)
        return fwd_page

    @autopilot.logging.log_action(logger.info)
    def unset_forward_on_busy(self, sim=None):
        """Disables forwarding when busy on a sim.

        :param sim: Number of what SIM to use, either 1 or 2.
            Required parameter in dual SIM setups
        :returns: The Call Forwarding settings page.
        """
        fwd_page = self._enter_call_forwarding(sim)
        fwd_page.unset_forward_on_busy()
        return fwd_page

    @autopilot.logging.log_action(logger.info)
    def get_forward_on_busy(self, sim=None):
        """Return forwarding on busy value on sim.

        :param sim: Number of what SIM to use, either 1 or 2.
            Required parameter in dual SIM setups
        :returns: The forward on busy value.
        """
        fwd_page = self._enter_call_forwarding(sim)
        return fwd_page.get_forward_on_busy()

    @autopilot.logging.log_action(logger.info)
    def set_forward_when_no_answer(self, number, sim=None):
        """Sets forwarding when no answer to number on a sim.

        :param sim: Number of what SIM to use, either 1 or 2.
            Required parameter in dual SIM setups
        :param number: Number to which we want to forward.
        :returns: The Call Forwarding settings page.
        """
        fwd_page = self._enter_call_forwarding(sim)
        fwd_page.set_forward_when_no_answer(number)
        return fwd_page

    @autopilot.logging.log_action(logger.info)
    def unset_forward_when_no_answer(self, sim=None):
        """Disables forwarding when no answer on a sim.

        :param sim: Number of what SIM to use, either 1 or 2.
            Required parameter in dual SIM setups
        :returns: The Call Forwarding settings page.
        """
        fwd_page = self._enter_call_forwarding(sim)
        fwd_page.unset_forward_when_no_answer()
        return fwd_page

    @autopilot.logging.log_action(logger.info)
    def get_forward_when_no_answer(self, sim=None):
        """Return forwarding when no answer value on sim.

        :param sim: Number of what SIM to use, either 1 or 2.
            Required parameter in dual SIM setups
        :returns: The forward when no answer value.
        """
        fwd_page = self._enter_call_forwarding(sim)
        return fwd_page.get_forward_when_no_answer()

    @autopilot.logging.log_action(logger.info)
    def set_forward_when_unreachable(self, number, sim=None):
        """Sets forwarding when unreachable to number on a sim.

        :param sim: Number of what SIM to use, either 1 or 2.
            Required parameter in dual SIM setups
        :param number: Number to which we want to forward.
        :returns: The Call Forwarding settings page.
        """
        fwd_page = self._enter_call_forwarding(sim)
        fwd_page.set_forward_when_unreachable(number)
        return fwd_page

    @autopilot.logging.log_action(logger.info)
    def unset_forward_when_unreachable(self, sim=None):
        """Disables forwarding when unreachable on a sim.

        :param sim: Number of what SIM to use, either 1 or 2.
            Required parameter in dual SIM setups
        """
        fwd_page = self._enter_call_forwarding(sim)
        fwd_page.unset_forward_when_unreachable()
        return fwd_page

    @autopilot.logging.log_action(logger.info)
    def get_forward_when_unreachable(self, sim=None):
        """Return forwarding when unreachable value on sim.

        :param sim: Number of what SIM to use, either 1 or 2.
            Required parameter in dual SIM setups
        :returns: The forward when unreachable value.
        """
        fwd_page = self._enter_call_forwarding(sim)
        return fwd_page.get_forward_when_unreachable()


class CallWaiting(lomiriuitoolkit.LomiriUIToolkitCustomProxyObjectBase):

    """Autopilot helper for the Call waiting page."""

    @property
    def _switch(self):
        return self.wait_select_single(
            lomiriuitoolkit.CheckBox,
            objectName='callWaitingSwitch')

    def enable_call_waiting(self):
        self._switch.check()

    def disable_call_waiting(self):
        self._switch.uncheck()


class CallForwarding(lomiriuitoolkit.LomiriUIToolkitCustomProxyObjectBase):

    """Autopilot helper for the Call forwarding page."""

    @autopilot.logging.log_action(logger.info)
    def _set_rule(self):
        """Saves rule."""
        button = self.wait_select_single('Button',
                                         objectName='setButton')
        self.pointing_device.click_object(button)

    @autopilot.logging.log_action(logger.info)
    def _check_rule(self, rule, value):
        """Checks a rule's associated CheckBox with value.

        :param rule: The string representation of the rule.
        :param value: The new value of the CheckBox.
        """
        check = self.wait_select_single(lomiriuitoolkit.CheckBox,
                                        objectName='check_%s' % rule)
        if value:
            check.check()
        else:
            check.uncheck()

    @autopilot.logging.log_action(logger.info)
    def enable_rule(self, rule):
        """Enables a rule.

        :param rule: The string representation of the rule.
        """
        self._check_rule(rule, True)

    @autopilot.logging.log_action(logger.info)
    def disable_rule(self, rule):
        """Disables a rule.

        :param rule: The string representation of the rule.
        """
        self._check_rule(rule, False)

    @autopilot.logging.log_action(logger.info)
    def _enter_rule(self, rule, number):
        """Enters a number into a rule's associated TextField.

        :param rule: The string representation of the rule.
        :param number: Number to enter into field.
        """
        field = self.wait_select_single('TextField',
                                        objectName='field_%s' % rule)
        field.write(number)

    @autopilot.logging.log_action(logger.info)
    def _get_rule(self, rule):
        """Returns the value of the rule.

        :param rule: The string representation of the rule.
        """
        return self.wait_select_single(objectName='current_%s' % rule).value

    @autopilot.logging.log_action(logger.info)
    def set_forward_unconditionally(self, number):
        """Sets forward unconditionally.

        :param number: Number to forward to.
        """
        self.enable_rule('voiceUnconditional')
        self._enter_rule('voiceUnconditional', number)
        self._set_rule()

    @autopilot.logging.log_action(logger.info)
    def unset_forward_unconditionally(self):
        """Disables forward unconditionally."""
        self.disable_rule('voiceUnconditional')

    @autopilot.logging.log_action(logger.info)
    def get_forward_unconditionally(self):
        """Returns value of forward unconditionally."""
        return self._get_rule('voiceUnconditional')

    @autopilot.logging.log_action(logger.info)
    def set_forward_on_busy(self, number):
        """Sets forward on busy.

        :param number: Number to forward to.
        """
        self.enable_rule('voiceBusy')
        self._enter_rule('voiceBusy', number)
        self._set_rule()

    @autopilot.logging.log_action(logger.info)
    def unset_forward_on_busy(self):
        """Disables forward on busy."""
        self.disable_rule('voiceBusy')

    @autopilot.logging.log_action(logger.info)
    def get_forward_on_busy(self):
        """Returns value of forward on busy."""
        return self._get_rule('voiceBusy')

    @autopilot.logging.log_action(logger.info)
    def set_forward_when_no_answer(self, number):
        """Sets forward on no answer.

        :param number: Number to forward to.
        """
        self.enable_rule('voiceNoReply')
        self._enter_rule('voiceNoReply', number)
        self._set_rule()

    @autopilot.logging.log_action(logger.info)
    def unset_forward_when_no_answer(self):
        """Disables forward when no answer."""
        self.disable_rule('voiceNoReply')

    @autopilot.logging.log_action(logger.info)
    def get_forward_when_no_answer(self):
        """Returns value of forward on no answer."""
        return self._get_rule('voiceNoReply')

    @autopilot.logging.log_action(logger.info)
    def set_forward_when_unreachable(self, number):
        """Sets forward when unreachable.

        :param number: Number to forward to.
        """
        self.enable_rule('voiceNotReachable')
        self._enter_rule('voiceNotReachable', number)
        self._set_rule()

    @autopilot.logging.log_action(logger.info)
    def unset_forward_when_unreachable(self):
        """Disables forward when unreachable."""
        self.disable_rule('voiceNotReachable')

    @autopilot.logging.log_action(logger.info)
    def get_forward_when_unreachable(self):
        """Returns value of forward when unreachable."""
        return self._get_rule('voiceNotReachable')


class Services(lomiriuitoolkit.LomiriUIToolkitCustomProxyObjectBase):

    """Autopilot helper for the Sim Services page."""

    # TODO: add pages for each relevant sim services page
    def open_sim_service(self, service):
        """Return a sim service page"""
        pass


class ResetPage(lomiriuitoolkit.LomiriUIToolkitCustomProxyObjectBase):

    """Autopilot helper for the Reset page."""

    @classmethod
    def validate_dbus_object(cls, path, state):
        name = introspection.get_classname_from_path(path)
        if name == b'PageComponent':
            if state['objectName'][1] == 'resetPage':
                return True
        return False

    @autopilot.logging.log_action(logger.info)
    def reset_launcher(self):
        """Reset the launcher.

        :returns: The main system settings page object, that will be visible
            after the reset is complete.

        """
        confirm_dialog = self._click_reset_launcher()
        confirm_dialog.confirm_reset()
        return self._wait_and_return_main_system_settins_page()

    @autopilot.logging.log_action(logger.debug)
    def _click_reset_launcher(self):
        button = self.select_single(objectName='resetLauncher')
        self.pointing_device.click_object(button)
        return self.get_root_instance().wait_select_single(
            objectName='resetLauncherDialog')

    def _wait_and_return_main_system_settins_page(self):
        main_view = self.get_root_instance().select_single(
            objectName='systemSettingsMainView')
        main_view.system_settings_page.active.wait_for(True)
        return main_view.system_settings_page

    @autopilot.logging.log_action(logger.info)
    def erase_and_reset_everything(self):
        """Reset to factory settings.

        :returns: The main system settings page object, that will be visible
            after the reset is complete.

        """
        confirm_dialog = self._click_factory_reset()
        confirm_dialog.confirm_reset()
        return self._wait_and_return_main_system_settins_page()

    @autopilot.logging.log_action(logger.debug)
    def _click_factory_reset(self):
        button = self.select_single(objectName='factoryReset')
        self.pointing_device.click_object(button)
        return self.get_root_instance().select_single(
            objectName='factoryResetDialog')


class ResetLauncherConfirmationDialog(
        lomiriuitoolkit.LomiriUIToolkitCustomProxyObjectBase):

    """Autopilot helper for the Reset Launcher Confirmation dialog."""

    @classmethod
    def validate_dbus_object(cls, path, state):
        name = introspection.get_classname_from_path(path)
        if name == b'Dialog':
            if state['objectName'][1] == 'resetLauncherDialog':
                return True
        return False

    @autopilot.logging.log_action(logger.debug)
    def confirm_reset(self):
        button = self.select_single('Button', objectName='resetLauncherAction')
        self.pointing_device.click_object(button)


class FactoryResetConfirmationDialog(
        lomiriuitoolkit.LomiriUIToolkitCustomProxyObjectBase):

    """Autopilot helper for the Reset Launcher Confirmation dialog."""

    @classmethod
    def validate_dbus_object(cls, path, state):
        name = introspection.get_classname_from_path(path)
        if name == b'Dialog':
            if state['objectName'][1] == 'factoryResetDialog':
                return True
        return False

    @autopilot.logging.log_action(logger.debug)
    def confirm_reset(self):
        button = self.select_single('Button', objectName='factoryResetAction')
        self.pointing_device.click_object(button)


class LanguagePage(lomiriuitoolkit.LomiriUIToolkitCustomProxyObjectBase):

    """Autopilot helper for the Language page."""

    @classmethod
    def validate_dbus_object(cls, path, state):
        name = introspection.get_classname_from_path(path)
        if name == b'PageComponent':
            if state['objectName'][1] == 'languagePage':
                return True
        return False

    def get_current_language(self):
        return self.wait_select_single(
            objectName='currentLanguage').currentLanguage

    def _click_change_display_language(self):
        item = self.select_single(objectName='displayLanguage')
        self.pointing_device.click_object(item)
        return self.get_root_instance().select_single(
            objectName='displayLanguageDialog')

    @autopilot.logging.log_action(logger.info)
    def change_display_language(self, langIndex, reboot=True):
        """Changes display language.

        :param langIndex: The language index to change to.

        :param reboot: Whether to reboot or not

        :returns: The language page

        """
        dialog = self._click_change_display_language()
        dialog.set_language(langIndex, reboot)
        return self.get_root_instance()


class DisplayLanguage(lomiriuitoolkit.LomiriUIToolkitCustomProxyObjectBase):

    """Autopilot helper for the Display Language dialog."""

    @autopilot.logging.log_action(logger.debug)
    def set_language(self, index, reboot):
        self._click_language_item(index)
        reboot_dialog = self._click_confirm()

        if reboot:
            reboot_dialog.reboot()
        else:
            reboot_dialog.revert()

    @autopilot.logging.log_action(logger.debug)
    def _click_language_item(self, index):
        languages_list = self.select_single('QQuickListView',
                                            objectName='languagesList')
        languages_list.click_element('languageName%d' % index)

    @autopilot.logging.log_action(logger.debug)
    def _click_confirm(self):
        button = self.select_single('Button',
                                    objectName='confirmChangeLanguage')
        self.pointing_device.click_object(button)
        return self.get_root_instance().select_single(
            objectName='rebootNecessaryDialog')

    @autopilot.logging.log_action(logger.debug)
    def _click_cancel(self):
        button = self.select_single('Button',
                                    objectName='cancelChangeLanguage')
        self.pointing_device.click_object(button)


class RebootNecessary(
        lomiriuitoolkit.LomiriUIToolkitCustomProxyObjectBase):

    """Autopilot helper for the Reboot Necessary dialog."""

    @autopilot.logging.log_action(logger.debug)
    def reboot(self):
        self._click_reboot()

    @autopilot.logging.log_action(logger.debug)
    def revert(self):
        self._click_revert()

    @autopilot.logging.log_action(logger.debug)
    def _click_reboot(self):
        button = self.select_single('Button', objectName='reboot')
        self.pointing_device.click_object(button)

    @autopilot.logging.log_action(logger.debug)
    def _click_revert(self):
        button = self.select_single('Button', objectName='revert')
        self.pointing_device.click_object(button)


class WifiPage(lomiriuitoolkit.LomiriUIToolkitCustomProxyObjectBase):

    """Autopilot helper for the WiFi page."""

    @classmethod
    def validate_dbus_object(cls, path, state):
        name = introspection.get_classname_from_path(path)
        if name == b'PageComponent':
            if state['objectName'][1] == 'wifiPage':
                return True
        return False

    @autopilot.logging.log_action(logger.debug)
    def enable_wireless(self):
        self._set_wireless(True)

    @autopilot.logging.log_action(logger.debug)
    def disable_wireless(self):
        self._set_wireless(False)

    """
    :returns: Whether or not WiFi can be used
    """
    @autopilot.logging.log_action(logger.debug)
    def have_wireless(self):
        try:
            self.wait_select_single('SwitchMenu', text=_('Wi-Fi'))
        except StateNotFoundError:
            return False
        return True

    """Returns the current WiFi state

    :returns: Whether or not WiFi is enabled
    """
    @autopilot.logging.log_action(logger.debug)
    def get_wireless(self):
        return self.wait_select_single(
            'SwitchMenu', text=_('Wi-Fi')).checked

    @autopilot.logging.log_action(logger.debug)
    def _set_wireless(self, state):
        obj = self.wait_select_single('SwitchMenu', text=_('Wi-Fi'))
        if obj.checked != state:
            self.pointing_device.click_object(obj)

    @autopilot.logging.log_action(logger.debug)
    def go_to_previous_networks(self):
        return self._click_previous_network()

    """Removes previous network

    :param ssid: Network name string (SSID)

    :returns: PreviousNetwork page

    """
    @autopilot.logging.log_action(logger.debug)
    def remove_previous_network(self, ssid):
        page = self.go_to_previous_networks()
        details = page.select_network(ssid)
        details.forget_network()
        return page

    @autopilot.logging.log_action(logger.debug)
    def _click_previous_network(self):

        # we can't mock the qunitymenu items, so we
        # have to wait for them to be built
        sleep(0.5)

        button = self.select_single('*',
                                    objectName='previousNetwork')
        self._scroll_to_and_click(button)
        return self.get_root_instance().wait_select_single(
            objectName='previousNetworksPage')


class OtherNetwork(
        lomiriuitoolkit.LomiriUIToolkitCustomProxyObjectBase):

    """Autopilot helper for the Connect to Hidden Network dialog."""

    @classmethod
    def validate_dbus_object(cls, path, state):
        name = introspection.get_classname_from_path(path)
        if name == b'Dialog':
            if state['objectName'][1] == 'otherNetworkDialog':
                return True
        return False

    @autopilot.logging.log_action(logger.debug)
    def enter_password(self, password):
        self._enter_password(password)

    @autopilot.logging.log_action(logger.debug)
    def _enter_password(self, password):
        pwdfield = self.wait_select_single('TextField',
                                           objectName='password')
        self._scroll_to_and_click(pwdfield)
        pwdfield.write(password)

    @autopilot.logging.log_action(logger.debug)
    def connect(self):
        self._click_connect()

    @autopilot.logging.log_action(logger.debug)
    def _click_connect(self):
        button = self.select_single('Button', objectName='connect')
        self._scroll_to_and_click(button)


class PreviousNetworks(
        lomiriuitoolkit.LomiriUIToolkitCustomProxyObjectBase):

    """Autopilot helper for the Previous Networks page."""

    @classmethod
    def validate_dbus_object(cls, path, state):
        name = introspection.get_classname_from_path(path)
        if name == b'ItemPage':
            if state['objectName'][1] == 'previousNetworksPage':
                return True
        return False

    @autopilot.logging.log_action(logger.debug)
    def select_network(self, name):
        self._select_network(name)
        return self.get_root_instance().wait_select_single(
            objectName='networkDetailsPage')

    @autopilot.logging.log_action(logger.debug)
    def _select_network(self, name):
        net = self.select_single('StandardProgression', text=name)
        self.pointing_device.click_object(net)


class NetworkDetails(
        lomiriuitoolkit.LomiriUIToolkitCustomProxyObjectBase):

    """Autopilot helper for the Networks Details page."""

    @autopilot.logging.log_action(logger.debug)
    def forget_network(self):
        self._click_forget()

    @autopilot.logging.log_action(logger.debug)
    def _click_forget(self):
        button = self.select_single('Button', objectName='forgetNetwork')
        self.pointing_device.click_object(button)


class VpnPage(lomiriuitoolkit.LomiriUIToolkitCustomProxyObjectBase):

    """Autopilot helper for the VPN page."""

    @classmethod
    def validate_dbus_object(cls, path, state):
        name = introspection.get_classname_from_path(path)
        if name == b'PageComponent':
            if state['objectName'][1] == 'vpnPage':
                return True
        return False

    @autopilot.logging.log_action(logger.debug)
    def add_vpn(self):
        obj = self.select_single(objectName='addVpnButton')
        self.get_root_instance().main_view.scroll_to_and_click(obj)
        return self.get_root_instance().wait_select_single(
            objectName='vpnEditor')

    @autopilot.logging.log_action(logger.debug)
    def preview_vpn(self, at):
        obj = self.wait_select_single(objectName='vpnListConnection%d' % at)
        self.get_root_instance().main_view.scroll_to_and_click(obj)
        return self.get_root_instance().wait_select_single(
            objectName='vpnPreviewDialog')

    @autopilot.logging.log_action(logger.debug)
    def change_vpn(self, at):
        diag = self.preview_vpn(at)
        change_button = diag.wait_select_single(
            objectName='vpnPreviewChangeButton'
        )
        self.get_root_instance().main_view.scroll_to_and_click(change_button)
        return self.get_root_instance().wait_select_single(
            objectName='vpnEditor')


class VpnEditor(
    lomiriuitoolkit.LomiriUIToolkitCustomProxyObjectBase
):
    """Autopilot helper for vpn change dialog."""

    @property
    def _openvpn_server_field(self):
        return self.wait_select_single(
            lomiriuitoolkit.TextField,
            objectName='vpnOpenvpnServerField')

    @property
    def _openvpn_port_field(self):
        return self.wait_select_single(
            lomiriuitoolkit.TextField,
            objectName='vpnOpenvpnPortField')

    @property
    def _openvpn_custom_port_toggle(self):
        return self.wait_select_single(
            lomiriuitoolkit.CheckBox,
            objectName='vpnOpenvpnCustomPortToggle')

    @property
    def _openvpn_tcp_toggle(self):
        return self.wait_select_single(
            lomiriuitoolkit.CheckBox,
            objectName='vpnOpenvpnTcpToggle')

    @property
    def _openvpn_udp_toggle(self):
        return self.wait_select_single(
            lomiriuitoolkit.CheckBox,
            objectName='vpnOpenvpnUdpToggle')

    @property
    def _openvpn_ca_field(self):
        return self.wait_select_single(objectName='vpnOpenvpnCaField')

    @property
    def _openvpn_ok_button(self):
        return self.wait_select_single(
            'Button', objectName='vpnEditorOkayButton')

    @autopilot.logging.log_action(logger.debug)
    def set_openvpn_server(self, server):
        self.get_root_instance().main_view.scroll_to(
            self._openvpn_server_field
        )
        self._openvpn_server_field.write(server)

    @autopilot.logging.log_action(logger.debug)
    def set_openvpn_custom_port(self, port):
        self.get_root_instance().main_view.scroll_to(
            self._openvpn_custom_port_toggle
        )
        self._openvpn_custom_port_toggle.check()
        # XXX: workaround for lp:1546559, i.e. we need to wait
        # some time between writing to the API.
        sleep(1)
        self._openvpn_port_field.write(port)

    @autopilot.logging.log_action(logger.debug)
    def set_openvpn_ca(self, paths):
        self.set_openvpn_file(self._openvpn_ca_field, paths)

    @autopilot.logging.log_action(logger.debug)
    def set_openvpn_file(self, field, paths):
        utils.dismiss_osk()
        self.get_root_instance().main_view.scroll_to_and_click(field)

        # Wait for expanded animation.
        sleep(0.5)

        # file = field.wait_select_single(objectName='vpnFileSelectorItem0')
        choose = field.wait_select_single(objectName='vpnFileSelectorItem1')
        self.pointing_device.click_object(choose)
        self.get_root_instance().main_view.scroll_to_and_click(choose)
        file_dialog = self.get_root_instance().wait_select_single(
            objectName='vpnDialogFile'
        )

        # Go to root /
        root = file_dialog.wait_select_single(objectName='vpnFilePathItem_/')
        self.pointing_device.click_object(root)

        for path in paths:
            list_view = file_dialog.wait_select_single(
                'QQuickListView', objectName='vpnFileList'
            )
            list_view.click_element('vpnFileItem_%s' % path)
        accept = file_dialog.wait_select_single(objectName='vpnFileAccept')
        self.pointing_device.click_object(accept)

    @autopilot.logging.log_action(logger.debug)
    def openvpn_okay(self):
        utils.dismiss_osk()
        self.get_root_instance().main_view.scroll_to_and_click(
            self._openvpn_ok_button
        )



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2013 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.

from __future__ import absolute_import

from fixtures import EnvironmentVariable
from testtools.matchers import NotEquals
from lomiri_system_settings.tests import SystemUpdatesBaseTestCase


class SystemUpdatesTestCases(SystemUpdatesBaseTestCase):

    """Tests for System Updates."""

    click_server_parameters = {
        'start': True
    }

    def setUp(self):
        # Set environment variables
        self.useFixture(
            EnvironmentVariable("IGNORE_CREDENTIALS", "True"))
        self.useFixture(
            EnvironmentVariable("URL_APPS", "http://0.0.0.0:9009/metadata"))
        super(SystemUpdatesTestCases, self).setUp()

    def test_check_for_updates_area(self):
        """Check that the updates area is shown on opening."""
        updates = self.system_settings.main_view.updates_page
        self.assertThat(updates, NotEquals(None))
        updates.stop_check()



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2014 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.

from __future__ import absolute_import

import dbus

from autopilot.matchers import Eventually
from testtools.matchers import Equals
from time import sleep
from lomiri_system_settings.tests import (WifiBaseTestCase,
                                          WifiWithSSIDBaseTestCase)
from lomiri_system_settings.utils.i18n import ugettext as _


class WifiEnabledTestCase(WifiBaseTestCase):
    """Tests for Language Page"""

    def test_wifi_page_title_is_correct(self):
        """Checks whether Wifi page is available"""
        self.assertThat(
            self.wifi_page.title,
            Equals(_('Wi-Fi')))

    def test_remove_previous_network(self):
        access_points = ['Series of Tubes', 'dev/null', 'Mi-Fi',
                         'MonkeySphere']

        for idx, ssid in enumerate(access_points):
            self.create_access_point('Mock_AP%d' % idx, ssid)
            self.obj_nm.AddWiFiConnection(
                self.device_path, 'Mock_Con%d' % idx, ssid, '',
                dbus.Dictionary(signature='sa{sv}')
            )

        self.wifi_page.remove_previous_network(access_points[0],)

        self.main_view.go_back()

        # wait for ui to update
        sleep(2)

        self.wifi_page.remove_previous_network(access_points[2],)

        # We cannot make any assertions, because connection deletion
        # is currently not supported.


class WifiDisabledTestCase(WifiBaseTestCase):

    indicatornetwork_parameters = {'actions': {
        'wifi.enable': (False, '', [False]),
    }}

    def test_connect_to_hidden_network_dialog_visibility(self):
        self.assertThat(
            lambda: bool(self.wifi_page.select_single(
                '*',
                objectName='connectToHiddenNetwork').visible),
            Eventually(Equals(False)), 'other net dialog not hidden')


class WifiWithTestSSIDTestCase(WifiWithSSIDBaseTestCase):

    ssid = 'test_ap'

    def test_handle_wifi_url_with_ssid(self):
        dialog = self.main_view.wait_select_single(
            objectName='otherNetworkDialog'
        )
        dialog._scroll_to_and_click = self.main_view.scroll_to_and_click
        dialog.enter_password('abcdefgh')
        dialog.connect()

        # allow backend to set up listeners
        sleep(0.3)

        if dialog:
            dialog.wait_until_destroyed()



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2014 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.

from __future__ import absolute_import

from autopilot.matchers import Eventually
from testtools.matchers import NotEquals

from lomiri_system_settings.tests import SoundBaseTestCase


class PhoneSoundTestCase(SoundBaseTestCase):
    """Tests for Phone Page"""

    def setUp(self):
        super(PhoneSoundTestCase, self).setUp(panel=None)
        self.page = self.main_view.go_to_phone_page()

    def test_dialpad_sounds_switch(self):
        """ Check that dialpad_sounds is present and clickable"""
        prev_value = self.obj_snd.GetDialpadSoundsEnabled()
        if not prev_value:
            self.page.enable_dialpad_sounds()
        else:
            self.page.disable_dialpad_sounds()
        self.assertThat(
            lambda: self.obj_snd.GetDialpadSoundsEnabled(),
            Eventually(NotEquals(prev_value)))



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2013 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.

from autopilot.matchers import Eventually
from gi.repository import Gio, GLib
from testtools.matchers import Equals
from lomiri_system_settings.tests import LomiriSystemSettingsTestCase

""" Tests for Lomiri System Settings """


class MainTestCase(LomiriSystemSettingsTestCase):
    """ Tests for Search """

    def setUp(self):
        super(MainTestCase, self).setUp()

    def test_enable_rotation_lock(self):
        gsettings = Gio.Settings.new('com.lomiri.touch.system')
        current = gsettings.get_value('rotation-lock').get_boolean()

        self.addCleanup(
            self.set_orientation, gsettings, GLib.Variant('b', current))

        self.main_view.disable_orientation_lock()
        self.main_view.enable_orientation_lock()

        self.assertThat(
            lambda: gsettings.get_value('rotation-lock').get_boolean(),
            Eventually(Equals(True)))

    def test_disable_rotation_lock(self):
        gsettings = Gio.Settings.new('com.lomiri.touch.system')
        current = gsettings.get_value('rotation-lock').get_boolean()

        self.addCleanup(
            self.set_orientation, gsettings, GLib.Variant('b', current))

        self.main_view.enable_orientation_lock()
        self.main_view.disable_orientation_lock()

        self.assertThat(
            lambda: gsettings.get_value('rotation-lock').get_boolean(),
            Eventually(Equals(False)))



'''ofonod D-BUS mock template'''

# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation; either version 3 of the License, or (at your option) any
# later version.  See http://www.gnu.org/copyleft/lgpl.html for the full text
# of the license.
import dbus
import dbusmock

__author__ = 'Martin Pitt'
__email__ = 'martin.pitt@ubuntu.com'
__copyright__ = '(c) 2013 Canonical Ltd.'
__license__ = 'LGPL 3+'

BUS_NAME = 'org.ofono'
MAIN_OBJ = '/'
MAIN_IFACE = 'org.ofono.Manager'
SYSTEM_BUS = True

NOT_IMPLEMENTED = '''raise dbus.exceptions.DBusException(
    "org.ofono.Error.NotImplemented")'''

_parameters = {}


def load(mock, parameters):
    global _parameters
    mock.modems = []  # object paths
    _parameters = parameters
    mock.AddMethod(MAIN_IFACE, 'GetModems', '', 'a(oa{sv})',
                   'ret = [(m, objects[m].GetAll(\
                        "org.ofono.Modem")) for m in self.modems]')

    if not parameters.get('no_modem', False):
        mock.AddModem(parameters.get('ModemName', 'ril_0'), {})


@dbus.service.method(dbusmock.MOCK_IFACE,
                     in_signature='sa{sv}', out_signature='s')
def AddModem(self, name, properties):
    '''Convenience method to add a modem

    You have to specify a device name which must be a valid part of an object
    path, e. g. "mock_ac". For future extensions you can specify a "properties"
    array, but no extra properties are supported for now.

    Returns the new object path.
    '''
    path = '/' + name
    self.AddObject(
        path,
        'org.ofono.Modem',
        {
            'Online': dbus.Boolean(True, variant_level=1),
            'Powered': dbus.Boolean(True, variant_level=1),
            'Lockdown': dbus.Boolean(False, variant_level=1),
            'Emergency': dbus.Boolean(False, variant_level=1),
            'Manufacturer': dbus.String('Fakesys', variant_level=1),
            'Model': dbus.String('Mock Modem', variant_level=1),
            'Revision': dbus.String('0815.42', variant_level=1),
            'Type': dbus.String('hardware', variant_level=1),
            'Interfaces': [
                'org.ofono.CallForwarding',
                'org.ofono.CallSettings',
                'org.ofono.CallVolume',
                'org.ofono.ConnectionManager',
                'org.ofono.NetworkRegistration',
                'org.ofono.RadioSettings',
                'org.ofono.SimManager',
                'org.ofono.VoiceCallManager',
                ],
            'Features': ['gprs', 'net'],
        },
        [
            ('GetProperties', '', 'a{sv}',
                'ret = self.GetAll("org.ofono.Modem")'),
            (
                'SetProperty', 'sv', '',
                'self.Set("org.ofono.Modem", args[0], args[1]); '
                'self.EmitSignal("org.ofono.Modem", "PropertyChanged",'
                '"sv", [args[0], args[1]])'),
        ])
    obj = dbusmock.mockobject.objects[path]
    obj.name = name
    obj.sim_pin = "2468"
    add_simmanager_api(obj)
    add_connectionmanager_api(obj)
    add_voice_call_api(obj)
    add_netreg_api(obj)
    self.modems.append(path)
    props = obj.GetAll('org.ofono.Modem', dbus_interface=dbus.PROPERTIES_IFACE)
    self.EmitSignal(MAIN_IFACE, 'ModemAdded', 'oa{sv}', [path, props])
    return path


def add_simmanager_api(mock):
    '''Add org.ofono.SimManager API to a mock'''

    iface = 'org.ofono.SimManager'
    mock.AddProperties(iface, {
        'CardIdentifier': _parameters.get('CardIdentifier', 12345),
        'Present': _parameters.get('Present', dbus.Boolean(True)),
        'SubscriberNumbers': _parameters.get('SubscriberNumbers',
                                             ['123456789', '234567890']),
        'SubscriberIdentity': _parameters.get('SubscriberIdentity', 23456),
        'ServiceNumbers': _parameters.get('ServiceNumbers', dbus.Dictionary(
            {
                'Fake Service': dbus.String('555-555', variant_level=1),
                'Faker Service': dbus.String('555-321', variant_level=1),
                'Fakest Service': dbus.String('555-123', variant_level=1)
            }, signature='sv'
        )),
        'LockedPins': _parameters.get('LockedPins', ['pin']),
        'Retries': _parameters.get('Retries', {'pin': dbus.Byte(3)}),
        'PinRequired': _parameters.get('PinRequired', 'none')
    })

    mock.AddMethods(iface, [
        ('GetProperties', '', 'a{sv}', 'ret = self.GetAll("%s")' % iface),
        ('SetProperty', 'sv', '', 'self.Set("%(i)s", args[0], args[1]); '
         'self.EmitSignal("%(i)s", "PropertyChanged", "sv", [args[0], '
         'args[1]])' % {'i': iface}),
        ('ResetPin', 'sss', '', '')
    ])


def add_connectionmanager_api(mock):
    '''Add org.ofono.ConnectionManager API to a mock'''

    iface = 'org.ofono.ConnectionManager'
    mock.contexts = []
    mock.AddProperties(iface, {
        'Attached': _parameters.get('Attached', True),
        'Bearer': _parameters.get('Bearer', 'gprs'),
        'RoamingAllowed': _parameters.get('RoamingAllowed', False),
        'Powered': _parameters.get('ConnectionPowered', True),
    })
    mock.AddMethods(iface, [
        ('GetProperties', '', 'a{sv}', 'ret = self.GetAll("%s")' % iface),
        ('SetProperty', 'sv', '', 'self.Set("%(i)s", args[0], args[1]); '
         'self.EmitSignal("%(i)s", "PropertyChanged", "sv", ['
         'args[0], args[1]])' % {'i': iface}),
        ('AddContext', 's', 'o', 'ret = self.AddConnectionContext(args[0])'),
        ('RemoveContext', 'o', '', 'self.RemoveConnectionContext(args[0])'),
        ('DeactivateAll', '', '', ''),
        ('GetContexts', '', 'a(oa{sv})', 'ret = self.GetConnectionContexts()'),
    ])

    interfaces = mock.GetProperties()['Interfaces']
    interfaces.append(iface)
    mock.SetProperty('Interfaces', interfaces)


@dbus.service.method('org.ofono.ConnectionManager',
                     in_signature='', out_signature='a(oa{sv})')
def GetConnectionContexts(self):
    contexts = dbus.Array([], signature='a(oa{sv})')
    for ctx in self.contexts:
        contexts.append(dbus.Struct(
            (ctx.__dbus_object_path__, ctx.GetProperties())))
    return contexts


@dbus.service.method('org.ofono.ConnectionManager',
                     in_signature='s', out_signature='o')
def AddConnectionContext(self, type):
    name = 'context%s' % str(len(self.contexts))
    path = '%s/%s' % (self.__dbus_object_path__, name)
    iface = 'org.ofono.ConnectionContext'

    # We give the context a name, just like ofono does.
    # See https://github.com/rilmodem/ofono/blob/master/src/gprs.c#L148
    ofono_default_accesspointname = {
        'internet': 'Internet',
        'mms': 'MMS',
        'ai': 'AI',
        'ims': 'IMS',
        'wap': 'WAP'
    }

    self.AddObject(
        path,
        iface,
        {
            'Active': False,
            'AccessPointName': '',
            'Type': type,
            'Username': '',
            'Password': '',
            'Protocol': 'ip',
            'Name': ofono_default_accesspointname[type],
            'Preferred': False,
            'Settings': dbus.Dictionary({}, signature='sv'),
            'IPv6.Settings': dbus.Dictionary({}, signature='sv'),
            'MessageProxy': '',
            'MessageCenter': '',
        },
        [
            ('GetProperties', '', 'a{sv}',
                'ret = self.GetAll("org.ofono.ConnectionContext")'),
            (
                'SetProperty', 'sv', '',
                'self.Set("%s", args[0], args[1]); '
                'self.EmitSignal("%s", "PropertyChanged",'
                '"sv", [args[0], args[1]])' % (iface, iface)),
        ])
    ctx_obj = dbusmock.get_object(path)
    self.contexts.append(ctx_obj)
    self.EmitSignal('org.ofono.ConnectionManager',
                    'ContextAdded', 'oa{sv}', [path, ctx_obj.GetProperties()])

    return path


@dbus.service.method('org.ofono.ConnectionManager',
                     in_signature='o', out_signature='')
def RemoveConnectionContext(self, path):
    ctx_obj = dbusmock.get_object(path)
    self.contexts.remove(ctx_obj)
    self.RemoveObject(path)
    self.EmitSignal('org.ofono.ConnectionManager',
                    'ContextRemoved', 'o', [path])


@dbus.service.method('org.ofono.SimManager',
                     in_signature='ss', out_signature='')
def LockPin(self, pin_type, pin):
    iface = 'org.ofono.SimManager'
    print('XXX LockPin', pin_type, pin)

    if (pin == self.sim_pin):
        print('XXX LockPin pin matches')
        self.Set(iface, "LockedPins", dbus.Array(["pin"]))
        self.EmitSignal(iface, "PropertyChanged", "sv",
                        ["LockedPins", self.Get(iface, "LockedPins")])
        self.Set(iface, "Retries", {'pin': dbus.Byte(3)})
        self.EmitSignal(iface, "PropertyChanged", "sv",
                        ["Retries", self.Get(iface, "Retries")])
    else:
        retries = self.Get(iface, "Retries")['pin']
        if (retries > 0):
            self.Set(iface, "Retries", {'pin': dbus.Byte(retries - 1)})
            self.EmitSignal(iface, "PropertyChanged", "sv",
                            ["Retries", self.Get(iface, "Retries")])
        raise dbus.exceptions.DBusException("", "Failed",
                                            name="org.ofono.Error.Failed")
    print('XXX LockPin', self.Get(iface, "Retries")['pin'])


@dbus.service.method('org.ofono.SimManager',
                     in_signature='ss', out_signature='')
def UnlockPin(self, pin_type, pin):
    iface = 'org.ofono.SimManager'
    print('XXX UnlockPin', pin_type, pin)

    if (pin == self.sim_pin):
        print('XXX UnlockPin pin matches')
        self.Set(iface, "LockedPins", "")
        self.EmitSignal(iface, "PropertyChanged", "sv",
                        ["LockedPins", self.Get(iface, "LockedPins")])
        self.Set(iface, "Retries", {'pin': dbus.Byte(3)})
        self.EmitSignal(iface, "PropertyChanged", "sv",
                        ["Retries", self.Get(iface, "Retries")])
    else:
        retries = self.Get(iface, "Retries")['pin']
        if (retries > 0):
            self.Set(iface, "Retries", {'pin': dbus.Byte(retries - 1)})
            self.EmitSignal(iface, "PropertyChanged", "sv",
                            ["Retries", self.Get(iface, "Retries")])
        raise dbus.exceptions.DBusException("", "Failed",
                                            name="org.ofono.Error.Failed")
    print('XXX UnlockPin', self.Get(iface, "Retries")['pin'])


@dbus.service.method('org.ofono.SimManager',
                     in_signature='sss', out_signature='')
def ChangePin(self, pin_type, pin, pin2):
    iface = 'org.ofono.SimManager'
    print('XXX ChangePin', pin_type, pin, pin2)

    if (pin == self.sim_pin):
        print('XXX ChangePin pin matches')
        self.sim_pin = pin2
        self.Set(iface, "Retries", {'pin': dbus.Byte(3)})
        self.EmitSignal(iface, "PropertyChanged", "sv",
                        ["Retries", self.Get(iface, "Retries")])
    else:
        retries = self.Get(iface, "Retries")['pin']
        if (retries > 0):
            self.Set(iface, "Retries", {'pin': dbus.Byte(retries - 1)})
            self.EmitSignal(iface, "PropertyChanged", "sv",
                            ["Retries", self.Get(iface, "Retries")])
        raise dbus.exceptions.DBusException("", "Failed",
                                            name="org.ofono.Error.Failed")


@dbus.service.method('org.ofono.SimManager',
                     in_signature='ss', out_signature='')
def EnterPin(self, pin_type, pin):
    iface = 'org.ofono.SimManager'
    print('XXX EnterPin', pin)

    if (pin == self.sim_pin):
        print('XXX EnterPin pin matches')
        self.Set(iface, "Retries", {'pin': dbus.Byte(3)})
        self.EmitSignal(iface, "PropertyChanged", "sv",
                        ["Retries", self.Get(iface, "Retries")])
    else:
        retries = self.Get(iface, "Retries")['pin']
        if (retries > 0):
            self.Set(iface, "Retries", {'pin': dbus.Byte(retries - 1)})
            self.EmitSignal(iface, "PropertyChanged", "sv",
                            ["Retries", self.Get(iface, "Retries")])
        raise dbus.exceptions.DBusException("", "Failed",
                                            name="org.ofono.Error.Failed")


def add_voice_call_api(mock):
    '''Add org.ofono.VoiceCallManager API to a mock'''

    # also add an emergency number which is not a real one, in case one runs a
    # test case against a production ofono :-)
    mock.AddProperty(
        'org.ofono.VoiceCallManager', 'EmergencyNumbers', ['911', '13373'])

    mock.calls = []  # object paths

    mock.AddMethods('org.ofono.VoiceCallManager', [
        (
            'GetProperties', '', 'a{sv}',
            'ret = self.GetAll("org.ofono.VoiceCallManager")'),
        ('Transfer', '', '', ''),
        ('SwapCalls', '', '', ''),
        ('ReleaseAndAnswer', '', '', ''),
        ('ReleaseAndSwap', '', '', ''),
        ('HoldAndAnswer', '', '', ''),
        ('SendTones', 's', '', ''),
        ('PrivateChat', 'o', 'ao', NOT_IMPLEMENTED),
        ('CreateMultiparty', '', 'o', NOT_IMPLEMENTED),
        ('HangupMultiparty', '', '', NOT_IMPLEMENTED),
        (
            'GetCalls', '', 'a(oa{sv})',
            'ret = [(c, objects[c].GetAll("org.ofono.VoiceCall"))\
             for c in self.calls]')
    ])


@dbus.service.method('org.ofono.VoiceCallManager',
                     in_signature='ss', out_signature='s')
def Dial(self, number, hide_callerid):
    path = self._object_path + '/voicecall%02i' % (len(self.calls) + 1)
    self.AddObject(
        path, 'org.ofono.VoiceCall',
        {
            'State': dbus.String('dialing', variant_level=1),
            'LineIdentification': dbus.String(number, variant_level=1),
            'Name': dbus.String('', variant_level=1),
            'Multiparty': dbus.Boolean(False, variant_level=1),
            'RemoteHeld': dbus.Boolean(False, variant_level=1),
            'RemoteMultiparty': dbus.Boolean(False, variant_level=1),
            'Emergency': dbus.Boolean(False, variant_level=1),
        },
        [
            (
                'GetProperties', '', 'a{sv}',
                'ret = self.GetAll("org.ofono.VoiceCall")'),
            ('Deflect', 's', '', NOT_IMPLEMENTED),
            (
                'Hangup', '', '',
                'self.parent.calls.remove(self._object_path);'
                'self.parent.RemoveObject(self._object_path);'
                'self.EmitSignal("org.ofono.VoiceCallManager",\
                     "CallRemoved", "o", [self._object_path])'),
            ('Answer', '', '', NOT_IMPLEMENTED),
        ])
    obj = dbusmock.mockobject.objects[path]
    obj.parent = self
    self.calls.append(path)
    self.EmitSignal('org.ofono.VoiceCallManager', 'CallAdded', 'oa{sv}',
                    [path, obj.GetProperties()])
    return path


@dbus.service.method('org.ofono.VoiceCallManager',
                     in_signature='', out_signature='')
def HangupAll(self):
    print('XXX HangupAll', self.calls)
    for c in list(self.calls):  # needs a copy
        dbusmock.mockobject.objects[c].Hangup()
    assert self.calls == []


def get_all_operators(mock):
    return 'ret = [(m, objects[m].GetAll("org.ofono.NetworkOperator")) ' \
           'for m in objects if "%s/operator/" in m]' % mock.name


def add_netreg_api(mock):
    '''Add org.ofono.NetworkRegistration API to a mock'''

    # also add an emergency number which is not a real one, in case one runs a
    # test case against a production ofono :-)
    mock.AddProperties('org.ofono.NetworkRegistration', {
        'Mode': 'auto',
        'Status': 'registered',
        'LocationAreaCode': _parameters.get('LocationAreaCode', 987),
        'CellId': _parameters.get('CellId', 10203),
        'MobileCountryCode': _parameters.get('MobileCountryCode', '777'),
        'MobileNetworkCode': _parameters.get('MobileNetworkCode', '11'),
        'Technology': _parameters.get('Technology', 'gsm'),
        'Name': _parameters.get('Name', 'fake.tel'),
        'Strength': _parameters.get('Strength', dbus.Byte(80)),
        'BaseStation': _parameters.get('BaseStation', ''),
    })

    mock.AddObject(
        '/%s/operator/op1' % mock.name,
        'org.ofono.NetworkOperator',
        {
            'Name': _parameters.get('Name', 'fake.tel'),
            'Status': 'current',
            'MobileCountryCode': _parameters.get('MobileCountryCode', '777'),
            'MobileNetworkCode': _parameters.get('MobileNetworkCode', '11'),
            'Technologies': [_parameters.get('Technology', 'gsm')],
        },
        [
            (
                'GetProperties', '', 'a{sv}',
                'ret = self.GetAll("org.ofono.NetworkOperator")'),
            ('Register', '', '', ''),
        ])

    mock.AddMethods('org.ofono.NetworkRegistration', [
        (
            'GetProperties', '', 'a{sv}',
            'ret = self.GetAll("org.ofono.NetworkRegistration")'),
        ('Register', '', '', ''),
        ('GetOperators', '', 'a(oa{sv})', get_all_operators(mock)),
        ('Scan', '', 'a(oa{sv})', get_all_operators(mock)),
    ])



'''connectivity D-BUS mock template'''

# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation; either version 3 of the License, or (at your option) any
# later version.  See http://www.gnu.org/copyleft/lgpl.html for the full text
# of the license.
import dbus
import dbusmock

__author__ = 'Jonas G. Drange'
__email__ = 'jonas.drange@canonical.com'
__copyright__ = '(c) 2015 Canonical Ltd.'
__license__ = 'LGPL 3+'

BUS_NAME = 'com.lomiri.connectivity1'
MAIN_IFACE = 'com.lomiri.connectivity1'
MAIN_OBJ = '/'
SYSTEM_BUS = False

PRIV_IFACE = 'com.lomiri.connectivity1.Private'
PRIV_OBJ = '/com/lomiri/connectivity1/Private'

MODEM_IFACE = 'com.lomiri.connectivity1.Modem'
SIM_IFACE = 'com.lomiri.connectivity1.Sim'

NETS_IFACE = 'com.lomiri.connectivity1.NetworkingStatus'
NETS_OBJ = '/com/lomiri/connectivity1/NetworkingStatus'

VPN_OBJ = '/com/lomiri/connectivity1/vpn'
VPN_CONN_IFACE = 'com.lomiri.connectivity1.vpn.VpnConnection'
VPN_CONN_OPENVPN_IFACE = 'com.lomiri.connectivity1.vpn.VpnConnection.OpenVpn'
VPN_CONN_PPTP_IFACE = 'com.lomiri.connectivity1.vpn.VpnConnection.Pptp'

NOT_IMPLEMENTED = '''raise dbus.exceptions.DBusException(
    "org.ofono.Error.NotImplemented")'''

_parameters = {}


def set_hotspot_enabled(self, value):
    self.SetProperty(NETS_OBJ, NETS_IFACE, 'HotspotEnabled', value)

    # Set HotspotStored = True if not stored and we're enabling it.
    stored = dbusmock.get_object(NETS_OBJ).Get(NETS_IFACE, 'HotspotStored')
    if value and not bool(stored):
        self.SetProperty(NETS_OBJ, NETS_IFACE, 'HotspotStored', True)


def set_hotspot_ssid(self, value):
    self.SetProperty(NETS_OBJ, NETS_IFACE, 'HotspotSsid', value)


def set_hotspot_password(self, value):
    self.SetProperty(PRIV_OBJ, PRIV_IFACE, 'HotspotPassword', value)


def set_hotspot_auth(self, value):
    self.SetProperty(PRIV_OBJ, PRIV_IFACE, 'HotspotAuth', value)


def set_wifi_enabled(self, value):
    self.SetProperty(NETS_OBJ, NETS_IFACE, 'WifiEnabled', value)


@dbus.service.method(dbusmock.MOCK_IFACE,
                     in_signature='s', out_signature='s')
def AddSim(self, iccid):
    path = "/com/lomiri/connectivity1/sim/{}".format(iccid)
    self.AddObject(
        path,
        SIM_IFACE,
        {
            'Iccid': dbus.String(iccid),
            'PrimaryPhoneNumber': dbus.String("358401234567"),
            'Locked': dbus.Boolean(False),
            'Present': dbus.Boolean(True),
            'Mcc': dbus.String("358"),
            'Mnc': dbus.String("42"),
            'PreferredLanguages': [dbus.String("en"), dbus.String("fi")],
            'DataRoamingEnabled': dbus.Boolean(False),
        },
        [
            ('Unlock', '', '', ''),
        ]
    )
    return path


@dbus.service.method(dbusmock.MOCK_IFACE,
                     in_signature='sis', out_signature='s')
def AddModem(self, serial, index, sim):
    path = "/com/lomiri/connectivity1/modem/{}".format(serial)
    self.AddObject(
        path,
        MODEM_IFACE,
        {
            'Index': dbus.Int32(index),
            'Serial': dbus.String(serial),
            'Sim': dbus.ObjectPath(sim)
        },
        [
        ]
    )
    return path


def add_openvpn_object(mock, path):
    obj = dbusmock.get_object(path)
    obj.AddProperties(VPN_CONN_OPENVPN_IFACE, {
        'connectionType': dbus.UInt32(0),
        'remote': dbus.String(),
        'ca': dbus.String(),
        'cert': dbus.String(),
        'certPass': dbus.String(),
        'key': dbus.String(),
        'username': dbus.String(),
        'password': dbus.String(),
        'localIp': dbus.String(),
        'remoteIp': dbus.String(),
        'staticKey': dbus.String(),
        'staticKeyDirection': dbus.UInt32(0),
        'portSet': dbus.Boolean(False),
        'port': dbus.UInt32(1194),
        'renegSecondsSet': dbus.Boolean(False),
        'renegSeconds': dbus.UInt32(0),
        'compLzo': dbus.Boolean(False),
        'protoTcp': dbus.Boolean(False),
        'devTypeSet': dbus.Boolean(False),
        'devType': dbus.UInt32(0),
        'dev': dbus.String(),
        'tunnelMtuSet': dbus.Boolean(False),
        'tunnelMtu': dbus.UInt32(1500),
        'fragmentSizeSet': dbus.Boolean(False),
        'fragmentSize': dbus.UInt32(1300),
        'mssFix': dbus.Boolean(False),
        'remoteRandom': dbus.Boolean(False),
        'cipher': dbus.UInt32(0),
        'keysizeSet': dbus.Boolean(False),
        'keysize': dbus.UInt32(128),
        'auth': dbus.UInt32(0),
        'tlsRemote': dbus.String(),
        'remoteCertTlsSet': dbus.Boolean(False),
        'remoteCertTls': dbus.UInt32(0),
        'taSet': dbus.Boolean(False),
        'ta': dbus.String(),
        'taDir': dbus.UInt32(0),
        'proxyType': dbus.UInt32(),
        'proxyServer': dbus.String(),
        'proxyPort': dbus.UInt32(80),
        'proxyRetry': dbus.Boolean(False),
        'proxyUsername': dbus.String(),
        'proxyPassword': dbus.String(),
    })


def add_pptp_object(mock, path):
    obj = dbusmock.get_object(path)
    obj.AddProperties(VPN_CONN_PPTP_IFACE, {
        'gateway': dbus.String(),
        'user': dbus.String(),
        'password': dbus.String(),
        'domain': dbus.String(),
        'allowPap': dbus.Boolean(True),
        'allowChap': dbus.Boolean(True),
        'allowMschap': dbus.Boolean(True),
        'allowMschapv2': dbus.Boolean(True),
        'allowEap': dbus.Boolean(True),
        'requireMppe': dbus.Boolean(False),
        'mppeType': dbus.UInt32(0),
        'mppeStateful': dbus.Boolean(False),
        'bsdCompression': dbus.Boolean(True),
        'deflateCompression': dbus.Boolean(True),
        'tcpHeaderCompression': dbus.Boolean(True),
        'sendPppEchoPackets': dbus.Boolean(False),
    })


def add_vpn_object(mock, vpn_type, path):
    mock.AddObject(
        path,
        VPN_CONN_IFACE,
        {
            'activatable': dbus.Boolean(True),
            'active': dbus.Boolean(False),
            'type': dbus.UInt32(vpn_type),
            'id': path.split('/')[len(path.split('/'))-1]
        },
        [
            ('UpdateSecrets', '', '', ''),
        ]
    )

    if vpn_type == 0:
        add_openvpn_object(mock, path)
    elif vpn_type == 1:
        add_pptp_object(mock, path)
    else:
        raise Exception("Unable to add vpn connection, no such type: %d" % (
            vpn_type)
        )


def add_vpn_connection(mock, vpn_type):
    conns = mock.Get(PRIV_IFACE, 'VpnConnections')
    new_path = '%s/%s%s' % (VPN_OBJ, 'MockVpnConnection', str(len(conns)))
    add_vpn_object(mock, vpn_type, new_path)
    conns.append(new_path)
    mock.SetProperty(PRIV_OBJ, PRIV_IFACE, 'VpnConnections', conns)
    return new_path


def remove_vpn_connection(mock, path):
    conns = mock.Get(PRIV_IFACE, 'VpnConnections')
    conns.remove(path)
    mock.SetProperty(PRIV_OBJ, PRIV_IFACE, 'VpnConnections', conns)


def load(mock, parameters):
    global _parameters
    _parameters = parameters

    mock.set_hotspot_enabled = set_hotspot_enabled
    mock.set_hotspot_ssid = set_hotspot_ssid
    mock.set_hotspot_password = set_hotspot_password
    mock.set_wifi_enabled = set_wifi_enabled
    mock.set_hotspot_auth = set_hotspot_auth
    mock.add_vpn_connection = add_vpn_connection
    mock.remove_vpn_connection = remove_vpn_connection

    mock.AddObject(
        NETS_OBJ,
        NETS_IFACE,
        {
            'HotspotSsid': _parameters.get(
                'HotspotSsid', dbus.ByteArray('Ubuntu'.encode('UTF-8'))),
            'HotspotEnabled': _parameters.get(
                'HotspotEnabled', dbus.Boolean(False)),
            'HotspotMode': _parameters.get('HotspotMode', dbus.String('ap')),
            'HotspotStored': _parameters.get(
                'HotspotStored', dbus.Boolean(False)
            ),
            'ModemAvailable': _parameters.get(
                'ModemAvailable', dbus.Boolean(True)
            ),
            'FlightModeSwitchEnabled': _parameters.get(
                'FlightModeSwitchEnabled', dbus.Boolean(False)
            ),
            'WifiSwitchEnabled': _parameters.get(
                'WifiSwitchEnabled', dbus.Boolean(False)
            ),
            'HotspotSwitchEnabled': _parameters.get(
                'HotspotSwitchEnabled', dbus.Boolean(False)
            ),
            'FlightMode': _parameters.get('FlightMode', dbus.Boolean(False)),
            'WifiEnabled': _parameters.get('WifiEnabled', dbus.Boolean(False)),
            # One of online, offline and connecting.
            'Status': _parameters.get('Status', 'offline')
        },
        []
    )

    mock.AddObject(
        PRIV_OBJ,
        PRIV_IFACE,
        {
            'HotspotPassword': _parameters.get(
                'HotspotPassword', dbus.String('abcdefgh')
            ),
            'HotspotAuth': _parameters.get(
                'HotspotAuth', dbus.String('wpa-psk')
            ),
            'VpnConnections': _parameters.get(
                'VpnConnections', dbus.Array([], signature='o')
            ),
            'MobileDataEnabled': _parameters.get(
                'MobileDataEnabled', dbus.Boolean(False)
            ),
            'SimForMobileData': _parameters.get(
                'SimForMobileData', dbus.ObjectPath('/')
            ),
            'Modems': _parameters.get(
                'Modems', dbus.Array([], signature='o')
            ),
            'Sims': _parameters.get(
                'Sims', dbus.Array([], signature='o')
            )
        },
        [
            (
                'UnlockAllModems', '', '',
                ''
            ),
            (
                'UnlockModem', 's', '',
                ''
            ),
            (
                'SetFlightMode', 'b', '',
                ''
            ),
            (
                'SetWifiEnabled', 'b', '',
                'objects["/"].set_wifi_enabled(self, args[0])'
            ),
            (
                'SetHotspotSsid', 'ay', '',
                'objects["/"].set_hotspot_ssid(self, args[0])'
            ),
            (
                'SetHotspotPassword', 's', '',
                'objects["/"].set_hotspot_password(self, args[0])'
            ),
            (
                'SetHotspotAuth', 's', '',
                'objects["/"].set_hotspot_auth(self, args[0])'
            ),
            (
                'SetHotspotEnabled', 'b', '',
                'objects["/"].set_hotspot_enabled(self, args[0])'
            ),
            (
                'SetHotspotMode', 's', '',
                ''
            ),
            (
                'AddVpnConnection', 'u', 'o',
                'ret = objects["/"].add_vpn_connection(self, args[0])'
            ),
            (
                'RemoveVpnConnection', 'o', '',
                'objects["/"].remove_vpn_connection(self, args[0])'
            )
        ]
    )


@dbus.service.method(dbusmock.MOCK_IFACE,
                     in_signature='sssv', out_signature='')
def SetProperty(self, path, iface, name, value):
    obj = dbusmock.get_object(path)
    obj.Set(iface, name, value)
    obj.EmitSignal(iface, 'PropertiesChanged', 'a{sv}', [{name: value}])



'''indicator-network D-BUS mock template'''

# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation; either version 3 of the License, or (at your option) any
# later version.  See http://www.gnu.org/copyleft/lgpl.html for the full text
# of the license.

import dbus
import dbusmock

__author__ = 'Jonas G. Drange'
__email__ = 'jonas.drange@canonical.com'
__copyright__ = '(c) 2015 Canonical Ltd.'
__license__ = 'LGPL 3+'

BUS_NAME = 'com.lomiri.indicator.network'
MAIN_IFACE = 'org.gtk.Actions'
MAIN_OBJ = '/com/lomiri/indicator/network'
MENU_IFACE = 'org.gtk.Menus'
PHONE_WIFI_OBJ = '/com/lomiri/indicator/network/phone_wifi_settings'
SYSTEM_BUS = False

NOT_IMPLEMENTED = '''raise dbus.exceptions.DBusException(
    "org.ofono.Error.NotImplemented")'''

_parameters = {}


def activate(self, action_name, parameters, platform_data):
    pass


def describe(self, action_name):
    return self.actions[action_name]


def describe_all(self):
    return self.actions


def list_actions(self):
    return list(self.actions)


def start(self, groups):
    return dbusmock.get_object(MAIN_OBJ).menus


def end(self, groups):
    pass


@dbus.service.method(dbusmock.MOCK_IFACE,
                     in_signature='asa{sb}a{sv}a{s(bgav)}', out_signature='')
def Changes(self, removals, enable_changes, state_changes, additions):
    obj = dbusmock.get_object(MAIN_OBJ)
    obj.EmitSignal(MAIN_IFACE, 'Changed', 'asa{sb}a{sv}a{s(bgav)}', [
        removals, enable_changes, state_changes, additions
    ])
    pass


def load(mock, parameters):
    global _parameters
    _parameters = parameters

    mock.describe = describe
    mock.describe_all = describe_all
    mock.list_actions = list_actions

    mock.actions = parameters.get('actions', {
        'wifi.enable': (True, '', [True]),
        'accesspoint.0': (True, '', [True]),
        'accesspoint.0::strength': (True, '', [44]),
        'accesspoint.1': (True, '', [False]),
        'accesspoint.1::strength': (True, '', [100]),
        'accesspoint.2': (True, '', [False]),
        'accesspoint.2::strength': (True, '', [74]),
        'accesspoint.3': (False, '', [False]),
    })

    mock.menus = parameters.get('menus', dbus.Array([
        (
            dbus.UInt32(0), dbus.UInt32(0),
            [
                {
                    'action': 'indicator.wifi.enable',
                    'x-ayatana-type': 'org.ayatana.indicator.switch',
                    'label': 'Wi-Fi'
                },
                {
                    'x-ayatana-type': 'org.ayatana.indicator.section',
                    'label': 'Available Wi-Fi networks'
                },
                {
                    ':section': dbus.Struct(
                        (dbus.UInt32(0), dbus.UInt32(1)), signature='(uu)'
                    )
                }
            ]
        ),
        (
            dbus.UInt32(0), dbus.UInt32(1),
            [
                {
                    'x-ayatana-wifi-ap-is-secure': True,
                    'x-ayatana-wifi-ap-is-enterprise': False,
                    'label': 'Secure',
                    'x-ayatana-type':
                        'unity.widgets.systemsettings.tablet.accesspoint',
                    'x-ayatana-wifi-ap-strength-action':
                        'indicator.accesspoint.0::strength',
                    'action': 'indicator.accesspoint.0',
                    'x-ayatana-wifi-ap-is-adhoc': False
                },
                {
                    'x-ayatana-wifi-ap-is-secure': False,
                    'x-ayatana-wifi-ap-is-enterprise': False,
                    'label': 'Insecure',
                    'x-ayatana-type':
                        'unity.widgets.systemsettings.tablet.accesspoint',
                    'x-ayatana-wifi-ap-strength-action':
                        'indicator.accesspoint.1::strength',
                    'action': 'indicator.accesspoint.1',
                    'x-ayatana-wifi-ap-is-adhoc': False
                },
                {
                    'x-ayatana-wifi-ap-is-secure': True,
                    'x-ayatana-wifi-ap-is-enterprise': True,
                    'label': 'Enterprise',
                    'x-ayatana-type':
                        'unity.widgets.systemsettings.tablet.accesspoint',
                    'x-ayatana-wifi-ap-strength-action':
                        'indicator.accesspoint.2::strength',
                    'action': 'indicator.accesspoint.2',
                    'x-ayatana-wifi-ap-is-adhoc': False
                },
                {
                    'x-ayatana-wifi-ap-is-secure': False,
                    'x-ayatana-wifi-ap-is-enterprise': False,
                    'label': 'Unknown',
                    'x-ayatana-type':
                        'unknown-type',
                    'action': 'indicator.accesspoint.3',
                    'x-ayatana-wifi-ap-is-adhoc': False
                }
            ]
        )
    ], signature='a(uuaa{sv})'))

    mock.AddMethods(
        MAIN_IFACE,
        [
            (
                'Activate', 'sava{sv}', '', ''
            ),
            (
                'Describe', 's', '(bgav)',
                'ret = self.describe(self, args[0])'
            ),
            (
                'DescribeAll', '', 'a{s(bgav)}',
                'ret = self.describe_all(self)'
            ),
            (
                'List', '', 'as',
                'ret = self.list_actions(self)'
            )
        ])

    mock.AddObject(
        PHONE_WIFI_OBJ,
        MENU_IFACE, {}, [
            (
                'Start', 'au', 'a(uuaa{sv})',
                'ret = self.start(self, args[0])'
            ),
            (
                'End', 'au', '',
                'ret = self.end(self, args[0])'
            )
        ]
    )

    phone_wifi_obj = dbusmock.get_object(PHONE_WIFI_OBJ)
    phone_wifi_obj.start = start
    phone_wifi_obj.end = end



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2014 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.

from __future__ import absolute_import

import os

from testtools.matchers import Equals

from lomiri_system_settings.tests import BackgroundBaseTestCase
from lomiri_system_settings.utils.i18n import ugettext as _


def get_wallpapers_from_grid(grid):
    return grid.select_many(objectName='itemImg')


class BackgroundTestCase(BackgroundBaseTestCase):
    """ Tests for Background Page """

    def setUp(self):
        super(BackgroundTestCase, self).setUp()
        self.background_page = self.main_view.background_page

    def get_wallpapergrid(self, name):
        """Return a WallpaperGrid with given name, or
        all of them"""
        return self.select_single('WallpaperGrid', objectName=name)

    def get_wallpapers(self, name=None):
        """Return individual wallpapers (QQuickImage) in given grid,
        or all of them"""
        if name:
            return get_wallpapers_from_grid(self.get_wallpapergrid(name))
        else:
            return self.all_wallpapers

    @property
    def all_wallpapergrids(self):
        """Return all WallpaperGrids"""
        return self.background_page.select_many('WallpaperGrid')

    @property
    def all_wallpapers(self):
        """Return all wallpapers in all grids"""
        wallpapers = []
        for grid in self.all_wallpapergrids:
            wallpapers.extend(get_wallpapers_from_grid(grid))
        return wallpapers

    @property
    def selected_wallpaper(self):
        """Return the currently selected QQuickImage.
        We grab the orange border and traverse a bit to get to this image"""
        selected_shape = self.background_page.select_single(
            objectName='highLight', visible=True)

        return selected_shape.get_parent().select_single(
            'QQuickImage', objectName='itemImg')

    def save_wallpaper(self):
        """Click on Set/Save button when previewing a wallpaper"""
        save = self.main_view.wait_select_single(
            objectName='saveButton')
        self.main_view.scroll_to_and_click(save)

    def test_background_page_title_is_correct(self):
        """ Checks whether Background page is available """
        self.assertThat(self.background_page.title, Equals(_('Background')))

    def test_change_background(self):
        """Test happy path for changing background"""

        # wallpaper source that is selected now
        old = self.selected_wallpaper.source

        # click a wallpaper that is not selected
        self.main_view.scroll_to_and_click(
            self.all_wallpapers[3])

        # click set/save
        self.save_wallpaper()

        # the newly selected wallpaper source
        new = self.selected_wallpaper.source

        # assert that UI is updated
        self.assertNotEqual(new, old)

        # assert that dbus changed
        dbus_value = "file://%s" % self.user_proxy.GetBackgroundFile()
        self.assertEqual(dbus_value, new)

    def test_that_the_currently_selected_background_comes_from_dbus(self):
        """Test that background file from dbus is selected in UI"""
        current_file = self.selected_wallpaper.source

        dbus_file = os.path.realpath(self.user_proxy.GetBackgroundFile())
        dbus_file = 'file://%s' % dbus_file

        self.assertEqual(current_file, dbus_file)

    def test_expand_collapse_custom(self):
        """Test that clicking the custom header changes its state"""
        custom = self.background_page.select_single(
            objectName='customArtGrid')
        custom_header = self.background_page.select_single(
            objectName='CustomHeader')

        self.assertEqual(custom.state, 'collapsed')
        self.main_view.scroll_to_and_click(custom_header)
        self.assertEqual(custom.state, '')



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2014 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.

from autopilot.matchers import Eventually
from testtools.matchers import Equals

from lomiri_system_settings.tests import VpnBaseTestCase

from lomiri_system_settings.tests.connectivity import (
    PRIV_IFACE as CTV_PRIV_IFACE, VPN_CONN_OPENVPN_IFACE
)


class VpnAddTestCase(VpnBaseTestCase):

    connectivity_parameters = {}

    def test_add_and_configure_openvpn(self):
        page = self.vpn_page.add_vpn()

        # Wait for length of VpnConnections to become 1
        self.assertThat(
            lambda: len(
                self.ctv_private.Get(CTV_PRIV_IFACE, 'VpnConnections')
            ),
            Eventually(Equals(1))
        )
        conn_path = self.ctv_private.Get(CTV_PRIV_IFACE, 'VpnConnections')[0]
        conn_obj = self.get_vpn_connection_object(conn_path)

        page.set_openvpn_server('vpn.ubuntu.com')
        page.set_openvpn_custom_port('1000')

        page.set_openvpn_ca(
            # Any file will do.
            ['etc', 'apt', 'sources.list']
        )
        page.openvpn_okay()

        self.assertThat(
            lambda: conn_obj.Get(VPN_CONN_OPENVPN_IFACE, 'remote'),
            Eventually(Equals('vpn.ubuntu.com'))
        )

        self.assertThat(
            lambda: conn_obj.Get(VPN_CONN_OPENVPN_IFACE, 'ca'),
            Eventually(Equals('/etc/apt/sources.list'))
        )

        self.assertThat(
            lambda: conn_obj.Get(VPN_CONN_OPENVPN_IFACE, 'portSet'),
            Eventually(Equals(True))
        )
        self.assertThat(
            lambda: conn_obj.Get(VPN_CONN_OPENVPN_IFACE, 'port'),
            Eventually(Equals(1000))
        )



from lomiriuitoolkit._custom_proxy_objects._common \
    import is_maliit_process_running
from ubuntu_keyboard.emulators.keyboard import Keyboard


def dismiss_osk():
    if is_maliit_process_running():
        osk = Keyboard()
        osk.dismiss()



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2013 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.

from autopilot.introspection.dbus import StateNotFoundError
from autopilot.matchers import Eventually
from testtools.matchers import Equals, NotEquals, raises

from lomiri_system_settings.tests import (
    LomiriSystemSettingsTestCase,
    LomiriSystemSettingsUpowerTestCase,
    LomiriSystemSettingsBatteryTestCase,
    LomiriSystemSettingsHotspotTestCase
)
from lomiri_system_settings.utils.i18n import ugettext as _


class SystemSettingsTestCases(LomiriSystemSettingsTestCase):
    """ Tests for Lomiri System Settings """

    def setUp(self):
        super(SystemSettingsTestCases, self).setUp()

    def test_title(self):
        """ Checks whether the app title is correct """
        header = self.main_view.select_single(
            objectName='systemSettingsPage'
        )
        self.assertThat(header, NotEquals(None))
        self.assertThat(header.title, Eventually(Equals(_('System Settings'))))

    def test_search(self):
        """ Checks whether the Search box is available """
        search = self.main_view.select_single(
            objectName='searchTextField'
        )
        self.assertThat(search, NotEquals(None))

    def test_network_category(self):
        """ Checks whether the Network category is available """
        category = self.main_view.select_single(
            objectName='categoryGrid-network'
        )
        self.assertThat(category, NotEquals(None))
        self.assertThat(
            category.categoryName, Eventually(Equals(_('Network')))
        )

    def test_personal_category(self):
        """ Checks whether the Personal category is available """
        category = self.main_view.select_single(
            objectName='categoryGrid-personal'
        )
        self.assertThat(category, NotEquals(None))
        self.assertThat(
            category.categoryName, Eventually(Equals(_('Personal')))
        )

    def test_system_category(self):
        """ Checks whether the System category is available """
        category = self.main_view.select_single(
            objectName='categoryGrid-system'
        )
        self.assertThat(category, NotEquals(None))
        self.assertThat(category.categoryName, Eventually(Equals(_('System'))))

    def test_wifi_plugin(self):
        """ Checks whether the Wi-Fi plugin is available """
        plugin = self.main_view.select_single(
            objectName='entryComponent-wifi'
        )
        self.assertThat(plugin, NotEquals(None))

    def test_cellular_plugin(self):
        """ Checks whether the Cellunar plugin is available """
        plugin = self.main_view.select_single(
            objectName='entryComponent-cellular'
        )
        self.assertThat(plugin, NotEquals(None))

    def test_bluetooth_plugin(self):
        """ Checks whether the Bluetooth plugin is available """
        plugin = self.main_view.select_single(
            objectName='entryComponent-bluetooth'
        )
        self.assertThat(plugin, NotEquals(None))

    def test_background_plugin(self):
        """ Checks whether the Background plugin is available """
        plugin = self.main_view.select_single(
            objectName='entryComponent-background'
        )
        self.assertThat(plugin, NotEquals(None))

    def test_sound_plugin(self):
        """ Checks whether the Sound plugin is available """
        plugin = self.main_view.select_single(
            objectName='entryComponent-sound'
        )
        self.assertThat(plugin, NotEquals(None))

    def test_language_plugin(self):
        """ Checks whether the Language plugin is available """
        plugin = self.main_view.select_single(
            objectName='entryComponent-language'
        )
        self.assertThat(plugin, NotEquals(None))

    def test_accounts_plugin(self):
        """ Checks whether the Accounts plugin is available """
        plugin = self.main_view.select_single(
            objectName='entryComponent-online-accounts'
        )
        self.assertThat(plugin, NotEquals(None))

    def test_timedate_plugin(self):
        """ Checks whether the Time and Date plugin is available """
        plugin = self.main_view.select_single(
            objectName='entryComponent-time-date'
        )
        self.assertThat(plugin, NotEquals(None))

    def test_security_plugin(self):
        """ Checks whether the Security plugin is available """
        plugin = self.main_view.select_single(
            objectName='entryComponent-security-privacy'
        )
        self.assertThat(plugin, NotEquals(None))

    def test_updates_plugin(self):
        """ Checks whether the Updates plugin is available """
        plugin = self.main_view.select_single(
            objectName='entryComponent-system-update'
        )
        self.assertThat(plugin, NotEquals(None))

    def test_vpn(self):
        """ Checks whether the Vpn plugin is available """
        plugin = self.main_view.select_single(
            objectName='entryComponent-vpn'
        )
        self.assertThat(plugin, NotEquals(None))


class SystemSettingsUpowerTestCases(LomiriSystemSettingsUpowerTestCase):
    def setUp(self):
        super(SystemSettingsUpowerTestCases, self).setUp()

    def test_no_battery_plugin_without_battery(self):
        """ Checks whether the Battery plugin is not available
        as we have no battery """
        self.assertThat(lambda: self.main_view.select_single(
            objectName='entryComponent-battery'),
            raises(StateNotFoundError)
        )

    def test_battery_plugin_battery_hotplugging(self):
        """ Checks whether hotplugging a battery makes the panel visible """
        self.add_mock_battery()
        plugin = self.main_view.select_single(
            objectName='entryComponent-battery'
        )
        self.assertThat(plugin, NotEquals(None))


class SystemSettingsBatteryTestCases(LomiriSystemSettingsBatteryTestCase):
    def setUp(self):
        super(SystemSettingsBatteryTestCases, self).setUp()

    def test_battery_plugin(self):
        """ Checks whether the Battery plugin is available """
        plugin = self.main_view.select_single(
            objectName='entryComponent-battery'
        )
        self.assertThat(plugin, NotEquals(None))


class SystemSettingsHotspotTestCases(LomiriSystemSettingsHotspotTestCase):

    def test_hotspot_plugin(self):
        """ Checks whether the Hotspot plugin is available for supported device
        """
        plugin = self.main_view.select_single(
            objectName='entryComponent-hotspot'
        )
        self.assertThat(plugin, NotEquals(None))


class SystemSettingsHotspotUnsupportedTestCases(
        LomiriSystemSettingsHotspotTestCase):

    # TODO: remove device parameter once lp:1434591 has been resolved.
    systemimage_parameters = {'device': 'mako'}
    connectivity_parameters = {'ModemAvailable': False}

    def test_hotspot_plugin_hidden(self):
        """ Checks that the Hotspot plugin is not available
        as it is broken on mako, and when there are no modems available."""
        self.assertThat(lambda: self.main_view.select_single(
            objectName='entryComponent-hotspot'),
            raises(StateNotFoundError)
        )
